<?xml version="1.0" encoding="UTF-8"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:w="http://jcam.org.uk/editor/picklist/1.0" 
    exclude-result-prefixes="xs xsd" version="2.0">

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: David RR Webber, April, 2008 -->

    <!-- 

Purpose : Take a  CAM template and create CAM picklist from the structure
Original : 04/17/2008 
Initials : drrw 
CAM ver : 1.1 
Description : Part of a set of tools to ease alignment of XSD based validations and documentation 
and CAM templates. 
How to use :  The runMode and depth parameters are provided to control the level of detail output.  For really large structures you may prefer to selectively increase the level of detail - initial settings should withNoAttr and depth 4 or 5.
                      Subsequent runs can then increase detail - after re-generating examples using xsd2example filtered to exclude unwanted parts.
Updated - Inits - Change 
04/17/08 drrw Initial release 
05/11/08 drrw Upgrade to work with excludeTree, excludeElement and excludeAttribute.
07/20/08 drrw Add parameter to allow minimal output of only excluded items.
07/31/08 drrw Fix xpath matching for attributes.
07/31/08 drrw Allow false/true for excludesOnly parameter
08/31/09 drrw Change w namespace to non-NIEM.
-->
    <xsl:param name="runMode" select="'withAttr'"/>
    <xsl:param name="depth" select="9"/>
    <xsl:param name="excludesOnly"></xsl:param>
    
    <xsl:output indent="yes"/>
    
    <xsl:variable name="version">1.04</xsl:variable>
    <xsl:variable name="rules" select="//as:BusinessUseContext/as:Rules/as:default/as:context"/>

    <xsl:template match="/">

        <xsl:variable name="ns-prefix">
            <xsl:value-of select="./@targetNamespace"/>
        </xsl:variable>     

        <w:WantList xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xmlns:w="http://jcam.org.uk/editor/picklist/1.0"
            xmlns:as="http://www.oasis-open.org/committees/cam"
            w:release="{$version}"
            w:product="NIEM/CAM" w:runMode="{$runMode}" w:depth="{$depth}"> 
            
            <xsl:comment>
                Generated with parameters -
                Mode: <xsl:value-of select="$runMode"/> 
                Depth: <xsl:value-of select="$depth"/> 
                Excludes: <xsl:value-of select="$excludesOnly"/> 
                Date: <xsl:value-of select="dateTime(current-date(),current-time())"/> 
                Version: <xsl:value-of select="$version"/>
            </xsl:comment>
            
            <xsl:for-each select="/as:CAM/as:AssemblyStructure/as:Structure/*">
                <!-- parses the entire XML input -->
                <xsl:element name="Group">
                    <xsl:attribute name="w:n"><xsl:value-of select="concat(name(../.),'\',name(.))"/></xsl:attribute>
                    <xsl:attribute name="w:isXclude">
                        <xsl:call-template name="chkExclude">
                            <xsl:with-param name="xpath" select="concat(name(../.),'/',name(.))"/>
                            <xsl:with-param name="type" select="'Group'"/>
                        </xsl:call-template>
                    </xsl:attribute>
                    <xsl:apply-templates select="./@*" mode="map"/>
                    <xsl:apply-templates mode="map" select="*"/>                  
                </xsl:element>
            </xsl:for-each>
           
        </w:WantList> 
    </xsl:template>
    
    <xsl:template match="node()/child::*" mode="map">

        <xsl:element name="Group">
            <xsl:attribute name="w:n"><xsl:value-of select="concat(name(../.),'\',name(.))"/></xsl:attribute>
            <xsl:attribute name="w:isXclude">
                <xsl:call-template name="chkExclude">
                    <xsl:with-param name="xpath" select="concat(name(../.),'/',name(.))"/>
                    <xsl:with-param name="type" select="'Group'"/>
                </xsl:call-template>
            </xsl:attribute>
            <xsl:apply-templates select="./@*" mode="map"/>
            <xsl:for-each select="*">
                <xsl:choose>
                    <xsl:when test="count(./descendant::node()) > 1">
                        <xsl:variable name="chkdepth">
                            <xsl:choose>
                                <xsl:when test="count(../../../../../../../parent::*) &gt; 0">7</xsl:when>
                                <xsl:when test="count(../../../../../../parent::*) &gt; 0">6</xsl:when>
                                <xsl:when test="count(../../../../../parent::*) &gt; 0">5</xsl:when>
                                <xsl:when test="count(../../../../parent::*) &gt; 0">4</xsl:when>
                                <xsl:when test="count(../../../parent::*) &gt; 0">3</xsl:when>
                                <xsl:when test="count(../../parent::*) &gt; 0">2</xsl:when>
                                <xsl:when test="count(../parent::*) &gt; 0">1</xsl:when>
                                <xsl:otherwise>8</xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:if test="$depth &gt; $chkdepth">
                            <xsl:apply-templates mode="map" select="."/>
                        </xsl:if>
                        <xsl:apply-templates mode="map" select="./@*"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:variable name="chk">
                            <xsl:call-template name="chkExclude">
                                <xsl:with-param name="xpath" select="concat(name(../../.),'/',name(../.),'/',name(.))"/>
                                <xsl:with-param name="type" select="'Element'"/>
                            </xsl:call-template>
                        </xsl:variable>
                        <xsl:if test="$excludesOnly='' or($excludesOnly='false') or($excludesOnly='true' and($chk='1'))">
                        <xsl:element name="Element">
                            <xsl:attribute name="w:n"><xsl:value-of select="concat(name(../../.),'\',name(../.),'\',name(.))"/></xsl:attribute>
                            <xsl:attribute name="w:isXclude"><!-- xsl:value-of select="'0'"/ -->
                                <xsl:value-of select="$chk"/>
                            </xsl:attribute>
                            <xsl:apply-templates  select="./@*" mode="map"/>
                        </xsl:element>
                        </xsl:if>
                    </xsl:otherwise>
                </xsl:choose>   
            </xsl:for-each>
         </xsl:element>
    </xsl:template>   
    
    <xsl:template match="attribute::node()" mode="map">
        <xsl:if test="contains($runMode,'withAttr')">
            <xsl:variable name="chk">
                <xsl:call-template name="chkExclude">
                    <xsl:with-param name="xpath" select="concat(name(../../.),'/',name(../.),'/@',name(.))"/>
                    <xsl:with-param name="type" select="'Attribute'"/>
                </xsl:call-template>
            </xsl:variable>
            <xsl:if test="$excludesOnly='' or($excludesOnly='false') or($excludesOnly='true' and($chk='1')) ">
            <xsl:element name="Attribute">
            <xsl:attribute name="w:n"><xsl:value-of select="concat(name(../../.),'\',name(../.),'\@',name(.))"/></xsl:attribute>
            <!-- xsl:attribute name="w:v"><xsl:value-of select="."/></xsl:attribute -->
            <xsl:attribute name="w:isXclude"><xsl:value-of select="$chk"/></xsl:attribute>
            </xsl:element>
            </xsl:if>
       </xsl:if>       
    </xsl:template>
    
    <xsl:template name="chkExclude">
        <xsl:param name="xpath"/>
        <xsl:param name="type"/>
       
        <xsl:choose>
            <xsl:when test="$type='Group'">
                <xsl:variable name="groupPath" select="$xpath"/>
                <xsl:variable name="shortPath" select="substring-after($xpath,'/')"/>
                <!-- xsl:message><xsl:value-of select="concat('excludeTree(//',$shortPath,')')"/></xsl:message -->
                <xsl:choose>
                    <xsl:when test="count($rules//as:constraint[@action=concat('excludeTree(//',$groupPath,')')]) &gt; 0">1</xsl:when>
                    <xsl:when test="count($rules//as:constraint[@action=concat('excludeTree(//',$shortPath,')')]) &gt; 0">1</xsl:when>
                    <xsl:otherwise>0</xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$type='Element'">
                <xsl:variable name="elementPath1" select="substring-after($xpath,'/')"/>
                <xsl:variable name="elementPath2" select="substring-after($elementPath1,'/')"/>
                <!-- xsl:message><xsl:value-of select="concat('excludeElement(//',$elementPath1,')')"/></xsl:message -->
                <!-- xsl:message><xsl:value-of select="concat('excludeElement(//',$elementPath2,')')"/></xsl:message -->
                <xsl:choose>
                    <xsl:when test="count($rules//as:constraint[@action=concat('excludeElement(//',$elementPath1,')')]) &gt; 0">1</xsl:when>
                    <xsl:when test="count($rules//as:constraint[@action=concat('excludeElement(//',$elementPath2,')')]) &gt; 0">1</xsl:when>
                    <xsl:otherwise>0</xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$type='Attribute'">
                <xsl:variable name="attrName" select="substring-after($xpath,'@')"/>
                <xsl:variable name="attrPath" select="substring-after($xpath,'/')"/>
                <!-- xsl:message>[<xsl:value-of select="$attrName"/>][<xsl:value-of select="$attrPath"/>]</xsl:message -->
                <xsl:choose>
                    <xsl:when test="count($rules//as:constraint[@action=concat('excludeAttribute(//@',$attrName,')')]) &gt; 0">1</xsl:when>
                    <xsl:when test="count($rules//as:constraint[@action=concat('excludeAttribute(//',$attrPath,')')]) &gt; 0">1</xsl:when>
                    <xsl:otherwise>0</xsl:otherwise>
                </xsl:choose>
                
            </xsl:when>
            <xsl:otherwise>0</xsl:otherwise>
        </xsl:choose>
     
    </xsl:template>

</xsl:stylesheet>
