<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:camed="http://jcam.org.uk/editor" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:as="http://www.oasis-open.org/committees/cam">

<!-- CAM utility functions -->
<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
     Authors: David RR Webber, June, 2009 -->
<!--
    Purpose  : Provide namespace mapping to output folder structure paths for cam-2-xsd.xsl program
    Original : 01/06/2010
    Initials : drrw
    CAM ver    : 1.1
    Description : Allows output mapping to output schema in file system folders depending on namespace details.
    Updated  - Inits - Change
    01/06/10   drrw    Initial release
    03/18/10   drrw    Add OASIS Standard mode
    03/29/10   drrw    Update OASIS standard details
    03/29/10   drrw    Fix handling of path offsets generation
    04/01/10   drrw    Add OASIS Draft mode
    04/05/10   drrw    Mapping of look ns to OASIS
    06/05/10   drrw   Relative path handling -for includes within includes it uses the primary path for that namespace
    06/08/10   drrw   Filename path contains filename stub in resolve ns outputpath
    12/11/10   drrw   Make root folder name for NIEM schemas instead of schema
    06/13/11   drrw   Move NIEM domain folders under domains folder
    06/13/11   drrw   Change copyright dates to 2011
    03/30/12   drrw   Resolve offset computation bug
-->

    <xsl:function name="camed:mapNS-2-path">
        <xsl:param name="ns"/>
        <xsl:param name="nsURI"/>
        <xsl:param name="mode"/>
        <xsl:param name="rules"/>
        <!-- xsl:message>Map: <xsl:value-of select="$ns"/> - <xsl:value-of select="$nsURI"/></xsl:message -->
        <xsl:variable name="constraintMatch">
            <xsl:variable name="Terms" select="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@type='Constraint']/concat(@URIcontains,'|')"/>
            <xsl:for-each select="$Terms">
            <xsl:for-each select="tokenize(.,'\|')">
                <xsl:variable name="this" select="."/>
                <!-- xsl:message>Constraint: <xsl:value-of select="."/></xsl:message -->
                <xsl:if test="string-length($this) &gt; 1 and(contains(lower-case($nsURI),$this))"><xsl:value-of select="$this"/>|<!-- xsl:message>this: <xsl:value-of select="$this"/></xsl:message --></xsl:if>
            </xsl:for-each>
            </xsl:for-each>
        </xsl:variable>

        <xsl:variable name="extensionMatch">
            <xsl:variable name="Terms" select="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@type='Extension']/concat(@URIcontains,'|')"/>
            <xsl:for-each select="$Terms">
            <xsl:for-each select="tokenize(.,'\|')">
                <!-- xsl:message>Extension: <xsl:value-of select="."/></xsl:message -->
                <xsl:variable name="this" select="."/>
                <xsl:if test="string-length($this) &gt; 1 and(contains(lower-case($nsURI),$this))"><xsl:value-of select="$this"/>|<!-- xsl:message>this: <xsl:value-of select="$this"/></xsl:message --></xsl:if>
            </xsl:for-each>
            </xsl:for-each>
        </xsl:variable>

        <!-- Rules precedence
            External - match on namespace
            Constraint - matching word constraint in uri
            Subset - match on namespace
            Extension - contains uri niem, or extn, or extension and not in subset list above
            Exchange - default output if not in above (exchange path can be simply blank)
        -->

        <xsl:choose><!-- now return the file name path depending on mode and type of schema -->
            <xsl:when test="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@prefix=$ns][@type='External']">
                <xsl:value-of select="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@prefix=$ns][@type='External'][1]"/>
            </xsl:when>
            <xsl:when test="string-length($constraintMatch) &gt; 0">
                <xsl:value-of select="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@URIcontains=substring-before($constraintMatch,'\|')][@type='Constraint']"/>
            </xsl:when>
            <xsl:when test="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@prefix=$ns][@type='Subset']">
                <xsl:value-of select="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@prefix=$ns][@type='Subset'][1]"/>
            </xsl:when>
            <xsl:when test="string-length($extensionMatch) &gt; 0">
                <xsl:variable name="matched">
                    <xsl:for-each select="tokenize($extensionMatch,'\|')">
                        <xsl:variable name="thisExtn" select="."/>
                        <xsl:value-of select="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@URIcontains=$thisExtn][@type='Extension']"/>
                    </xsl:for-each>
                </xsl:variable>
                <xsl:copy-of select="$matched[1]"/>
            </xsl:when>
            <xsl:otherwise><xsl:value-of select="$rules//as:xsd-mapping[@type=$mode]/as:map-ns[@type='Exchange'][1]"/></xsl:otherwise>
        </xsl:choose>

    </xsl:function>

    <xsl:function name="camed:resolve-ns-outputpath">
        <xsl:param name="fullPath"/>
        <xsl:param name="pathnameStub"/>
        <xsl:param name="nsOutputFolder"/>
        <xsl:param name="thisPathNodeOffset"/><!-- offset notation string value - ../../ etc -->

        <xsl:choose>
            <xsl:when test="string-length($thisPathNodeOffset) &gt; 0">
                    <xsl:value-of select="concat($thisPathNodeOffset,$pathnameStub)"/>
            </xsl:when>
            <xsl:when test="ends-with($fullPath,concat('\',$pathnameStub))">
                     <xsl:choose>
                         <xsl:when test="string-length($nsOutputFolder) &gt; 0">
                             <xsl:variable name="insertPath" select="if(ends-with($nsOutputFolder,'\')) then $nsOutputFolder else concat($nsOutputFolder,'\')"/>
                             <xsl:variable name="reversePath">
                                 <xsl:for-each select="reverse(tokenize($fullPath,'\\'))">
                                     <xsl:choose>
                                         <xsl:when test=".=$pathnameStub and(position() &lt;3)"/>
                                         <xsl:otherwise><xsl:value-of select="concat(.,'\')"/></xsl:otherwise>
                                     </xsl:choose>
                                 </xsl:for-each>
                             </xsl:variable>
                             <xsl:variable name="shortPath"><xsl:for-each select="reverse(tokenize($reversePath,'\\'))"><xsl:value-of select="if(position() &lt;2 and(.='')) then '' else concat(.,'\')"/></xsl:for-each>
                             </xsl:variable>
                             <xsl:value-of select="concat($shortPath,$insertPath,$pathnameStub)"/>
                         </xsl:when>
                         <xsl:otherwise><xsl:value-of select="$fullPath"/></xsl:otherwise>
                     </xsl:choose>
            </xsl:when>
            <xsl:when test="ends-with($fullPath,concat('/',$pathnameStub))">
                <xsl:choose>
                    <xsl:when test="string-length($nsOutputFolder) &gt; 0">
                        <xsl:variable name="insertPath" select="if(ends-with($nsOutputFolder,'/')) then $nsOutputFolder else concat($nsOutputFolder,'/')"/>
                        <xsl:variable name="reversePath">
                            <xsl:for-each select="reverse(tokenize($fullPath,'/'))">
                                <xsl:choose>
                                    <xsl:when test=".=$pathnameStub and(position() &lt;3)"/>
                                    <xsl:otherwise><xsl:value-of select="concat(.,'/')"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:for-each>
                        </xsl:variable>
                        <xsl:variable name="shortPath"><xsl:for-each select="reverse(tokenize($reversePath,'/'))"><xsl:value-of select="if(position() &lt;2 and(.='')) then '' else concat(.,'/')"/></xsl:for-each>
                        </xsl:variable>
                        <xsl:value-of select="concat($shortPath,$insertPath,$pathnameStub)"/>
                    </xsl:when>
                    <xsl:otherwise><xsl:value-of select="$fullPath"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="string-length($nsOutputFolder) &gt; 0">
                    <xsl:variable name="insertPath" select="if(ends-with($nsOutputFolder,'/')) then $nsOutputFolder else concat($nsOutputFolder,'/')"/>
                    <xsl:value-of select="concat(substring-before($fullPath,$pathnameStub),$insertPath,$pathnameStub)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="concat(substring-before($fullPath,$pathnameStub),$pathnameStub)"/>
            </xsl:otherwise>
        </xsl:choose>

    </xsl:function>

    <xsl:function name="camed:computeOffset">
        <xsl:param name="outputPath"/>
        <xsl:param name="targetPath"/>
        <xsl:choose>
            <xsl:when test="$outputPath=$targetPath"/>
            <xsl:otherwise>
                <!-- xsl:message> out: <xsl:value-of select="$outputPath"/> - target: <xsl:value-of select="$targetPath"/></xsl:message -->
                <xsl:variable name="outCount" select="count(tokenize($outputPath,'/'))"/>
                <xsl:variable name="targetCount" select="count(tokenize($targetPath,'/'))"/>
                <xsl:variable name="outpathFolders" select="tokenize($outputPath,'/')"/>
                <xsl:variable name="targetpathFolders" select="tokenize($targetPath,'/')"/>
                <xsl:variable name="howMuchMatches">
                    <xsl:for-each select="tokenize($targetPath,'/')">
                        <xsl:variable name="pos" select="position()"/>
                        <xsl:variable name="this" select="."/>
                        <xsl:choose>
                            <xsl:when test="$outpathFolders[$pos]=$this">1|</xsl:when>
                            <xsl:otherwise>0|</xsl:otherwise>
                        </xsl:choose>
                    </xsl:for-each>
                </xsl:variable>
                <xsl:variable name="matchCount" select="count(tokenize(substring-before($howMuchMatches,'0|'),'\|')) - 1"/>
                <!-- xsl:message> - matchCount <xsl:value-of select="$matchCount"/></xsl:message -->
                <xsl:variable name="resultPath">
                    <xsl:if test="$targetCount &lt; $outCount"><!-- shorter target tree so step back levels from current output -->
                        <xsl:for-each select="tokenize($outputPath,'/')"><xsl:if test="position() &gt; $targetCount">../</xsl:if></xsl:for-each>
                    </xsl:if>
                    <xsl:for-each select="tokenize($targetPath,'/')">
                        <xsl:variable name="pos" select="position()"/>
                        <xsl:variable name="this" select="."/>
                         <xsl:choose>
                             <xsl:when test="$pos = $targetCount"><xsl:if test="$outpathFolders[$pos - 1] = $targetpathFolders[$pos - 1]">../</xsl:if><xsl:value-of select="$this"/></xsl:when><!-- last leaf node -->
                             <xsl:when test="$pos &gt; $outCount"><xsl:if test="$outpathFolders[$pos - 1] = $targetpathFolders[$pos - 1]">../</xsl:if><xsl:value-of select="$this"/>/</xsl:when><!-- parent folder -->
                             <xsl:when test="$outpathFolders[$pos]=$this"/><!-- folder in from and including root is same -->
                             <xsl:when test="$pos &lt; $outCount + 1"><!-- path folders not equal and outpath is same length or shorter than target -->
                                 <xsl:choose>
                                     <xsl:when test="$pos = $outCount - 1"><!-- when we're immediately before the path end figure out offset -->
                                          <xsl:choose>
                                              <xsl:when test="$outCount - $matchCount &gt; 5 and($outpathFolders[$pos - 5] = $targetpathFolders[$pos - 5])">../../../../../../<xsl:value-of select="$this"/>/</xsl:when>
                                              <xsl:when test="$outCount - $matchCount &gt; 4 and($outpathFolders[$pos - 4] = $targetpathFolders[$pos - 4])">../../../../../<xsl:value-of select="$this"/>/</xsl:when>
                                              <xsl:when test="$outCount - $matchCount &gt; 3 and($outpathFolders[$pos - 3] = $targetpathFolders[$pos - 3])">../../../../<xsl:value-of select="$this"/>/</xsl:when>
                                              <xsl:when test="$outCount - $matchCount &gt; 2 and($outpathFolders[$pos - 2] = $targetpathFolders[$pos - 2])">../../../<xsl:value-of select="$this"/>/</xsl:when>
                                              <xsl:when test="$outCount - $matchCount &gt; 1 and($outpathFolders[$pos - 1] = $targetpathFolders[$pos - 1])">../../<xsl:value-of select="$this"/>/</xsl:when>
                                              <xsl:otherwise>../</xsl:otherwise>
                                          </xsl:choose>
                                     </xsl:when><!-- step back on non-matching path component -->
                                     <xsl:when test="$pos &lt; $outCount"/><!-- just loop for now -->
                                     <xsl:otherwise><xsl:if test="$outpathFolders[$pos - 1] = $targetpathFolders[$pos - 1]">../</xsl:if><xsl:value-of select="$this"/>/</xsl:otherwise>
                                 </xsl:choose>
                             </xsl:when>
                             <xsl:otherwise><xsl:if test="$outpathFolders[$pos - 1] = $targetpathFolders[$pos - 1]">../</xsl:if><xsl:value-of select="$this"/>/</xsl:otherwise><!-- step up one in path tree -->
                         </xsl:choose>
                    </xsl:for-each>
                </xsl:variable>
                <xsl:if test="$resultPath &gt; ''"><xsl:value-of select="replace(concat($resultPath,'/'),'//','/')"/></xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="camed:mode-comment-text">
        <xsl:param name="mode"/>
        <xsl:param name="style"/>
        <xsl:param name="ns-mappings"/>
        <xsl:choose>
            <xsl:when test="$ns-mappings/as:xsd-mappings/as:xsd-mapping[@type=$mode]">
                <xsl:value-of select="replace($ns-mappings/as:xsd-mappings/as:xsd-mapping[@type=$mode]/as:comment-hdr,'\$style',$style)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="replace($ns-mappings/as:xsd-mappings/as:xsd-mapping[@type='default']/as:comment-hdr,'\$style',$style)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="camed:mode-comment-end">
        <xsl:param name="mode"/>
        <xsl:param name="style"/>
        <xsl:param name="ns-mappings"/>
        <xsl:choose>
            <xsl:when test="$ns-mappings/as:xsd-mappings/as:xsd-mapping[@type=$mode]">
                <xsl:value-of select="replace($ns-mappings/as:xsd-mappings/as:xsd-mapping[@type=$mode]/as:comment-end,'\$style',$style)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="replace($ns-mappings/as:xsd-mappings/as:xsd-mapping[@type='default']/as:comment-end,'\$style',$style)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="camed:mode-comment-import">
        <xsl:param name="mode"/>
        <xsl:param name="filenamepath"/>
        <xsl:param name="ns"/>
        <xsl:param name="ns-mappings"/>
        <xsl:choose>
            <xsl:when test="$ns-mappings/as:xsd-mappings/as:xsd-mapping[@type=$mode]">
                <xsl:value-of select="replace(replace($ns-mappings/as:xsd-mappings/as:xsd-mapping[@type=$mode]/as:comment-import,'\$filenameStub',$filenamepath),'\$ns',$ns)"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="replace(replace($ns-mappings/as:xsd-mappings/as:xsd-mapping[@type='default']/as:comment-import,'\$filenameStub',$filenamepath),'\$ns',$ns)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="camed:InsertMappings">

        <xsl:variable name="mappings"><!-- Edit below here to change or add to text and folder path for mode and namespace mapping details. -->
            <as:xsd-mappings version="1.0" date="01/06/2011" xmlns:as="http://www.oasis-open.org/committees/cam">
                <as:xsd-mapping type="NIEM 2.1" date="01/06/2011">
                    <xsl:element name="as:comment-hdr" xml:space="preserve">

                        NIEM: $style schema - built from CAM template by XSLT
                        NIEM: Each namespace has its own import domain subset schema
                        NIEM: Reference schema is built separately
                    </xsl:element>
                    <xsl:element name="as:comment-end" xml:space="preserve">

                           NIEM: Note that this $style schema may be a subset from an original XSD schema.
                           NIEM: Please refer to the original source CAM template for updates and changes.
                           NIEM: For copyright, licensing and use criteria please refer to owner authority.
                    </xsl:element>
                    <xsl:element name="as:comment-import" xml:space="preserve">

                                    NIEM: subset domain schema referenced by exchange schema - $filenameStub
                                    NIEM: autogenerated from CAM template by XSLT
                                    NIEM: this schema is an import of type definitions for the main exchange schema
                                    NIEM: Namespace definitions for: $ns
                    </xsl:element>
                    <as:map-ns prefix="ansi-nist" type="Subset">Schemas/Subset/niem/ansi-nist/2.0</as:map-ns>
                    <as:map-ns prefix="apco" type="Subset">Schemas/Subset/niem/domains/apco/2.0</as:map-ns>
                    <as:map-ns prefix="atf" type="Subset">Schemas/Subset/niem/domains/atf/2.0</as:map-ns>
                    <as:map-ns prefix="can" type="Subset">Schemas/Subset/niem/domains/post-canada/2.0</as:map-ns>
                    <as:map-ns prefix="cap" type="Subset">Schemas/Subset/niem/domains/emergency/2.1</as:map-ns>
                    <as:map-ns prefix="cbrn" type="Subset">Schemas/Subset/niem/domains/cbrn/2.1</as:map-ns>
                    <as:map-ns prefix="cbrncl" type="Subset">Schemas/Subset/niem/domains/cbrncl/2.1</as:map-ns>
                    <as:map-ns prefix="census" type="Subset">Schemas/Subset/niem/domains/census/2.1</as:map-ns>
                    <as:map-ns prefix="de" type="Subset">Schemas/Subset/niem/domains/emergency/2.1</as:map-ns>
                    <as:map-ns prefix="dea" type="Subset">Schemas/Subset/niem/domains/dea/2.1</as:map-ns>
                    <as:map-ns prefix="edxl" type="Subset">Schemas/Subset/niem/domains/emergency/2.1</as:map-ns>
                    <as:map-ns prefix="edxl-cap" type="Subset">Schemas/Subset/niem/domains/emergency/2.1</as:map-ns>
                    <as:map-ns prefix="edxl-de" type="Subset">Schemas/Subset/niem/domains/emergency/2.1</as:map-ns>
                    <as:map-ns prefix="em" type="Subset">Schemas/Subset/niem/domains/emergency/2.1</as:map-ns>
                    <as:map-ns prefix="fbi" type="Subset">Schemas/Subset/niem/domains/fbi/2.1</as:map-ns>
                    <as:map-ns prefix="fs" type="Subset">Schemas/Subset/niem/domains/family/2.1</as:map-ns>
                    <as:map-ns prefix="geo" type="Subset">Schemas/Subset/niem/geo/2.1</as:map-ns>
                    <as:map-ns prefix="gml" type="Extension">Schemas/Extension/gml</as:map-ns>
                    <as:map-ns prefix="i" type="Subset">Schemas/Subset/niem/appinfo/2.1</as:map-ns>
                    <as:map-ns prefix="im" type="Subset">Schemas/Subset/niem/domains/immigration/2.1</as:map-ns>
                    <as:map-ns prefix="intel" type="Subset">Schemas/Subset/niem/domains/intel/2.1</as:map-ns>
                    <as:map-ns prefix="iso_3166" type="Subset">Schemas/Subset/niem/iso_3166/2.0</as:map-ns>
                    <as:map-ns prefix="ip" type="Subset">Schemas/Subset/niem/domains/infrapro/2.1</as:map-ns>
                    <as:map-ns prefix="it" type="Subset">Schemas/Subset/niem/domains/trade/2.1</as:map-ns>
                    <as:map-ns prefix="j" type="Subset">Schemas/Subset/niem/domains/jxdm/4.1</as:map-ns>
                    <as:map-ns prefix="m" type="Subset">Schemas/Subset/niem/domains/maritime/2.1</as:map-ns>
                    <as:map-ns prefix="nc" type="Subset">Schemas/Subset/niem/niem-core</as:map-ns>
                    <as:map-ns prefix="niem-xsd" type="Subset">Schemas/Subset/niem/proxy/xsd/2.0</as:map-ns>
                    <as:map-ns prefix="om" type="Subset">Schemas/Subset/niem/domains/om/2.1</as:map-ns>
                    <as:map-ns prefix="s" type="Subset">Schemas/Subset/niem/structures</as:map-ns>
                    <as:map-ns prefix="sar" type="Subset">Schemas/Subset/niem/domains/sar/2.1</as:map-ns>
                    <as:map-ns prefix="scr" type="Subset">Schemas/Subset/niem/domains/scr/2.1</as:map-ns>

                    <as:map-ns URIcontains="constraint" type="Constraint">Schemas/Constraint</as:map-ns>
                    <as:map-ns URIcontains="extn" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns URIcontains="extension" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns URIcontains="niem" type="Extension">Schemas/Extension</as:map-ns>

                    <as:map-ns default="true" type="Exchange">Schemas/Exchange</as:map-ns>

                    <as:map-ns prefix="xlink" type="External">Schemas/External/xlink</as:map-ns>
                    <as:map-ns prefix="xpil" type="External">Schemas/External/ciq3.0</as:map-ns>
                    <as:map-ns prefix="axal" type="External">Schemas/External/ciq3.0</as:map-ns>
                    <as:map-ns prefix="nxnl" type="External">Schemas/External/ciq3.0</as:map-ns>
                    <as:map-ns prefix="gml" type="External">Schemas/External/gml</as:map-ns>
                    <as:map-ns prefix="look" type="External">Schemas/External/cam</as:map-ns>
                </as:xsd-mapping>
                <as:xsd-mapping type="NIEM 2.0" date="01/06/2011">
                    <xsl:element name="as:comment-hdr" xml:space="preserve">

                        NIEM: $style schema - built from CAM template by XSLT
                        NIEM: Each namespace has its own import domain subset schema
                        NIEM: Reference schema is built separately
                    </xsl:element>
                    <xsl:element name="as:comment-end" xml:space="preserve">

                           NIEM: Note that this $style schema may be a subset from an original XSD schema.
                           NIEM: Please refer to the original source CAM template for updates and changes.
                           NIEM: For copyright, licensing and use criteria please refer to owner authority.
                    </xsl:element>
                    <xsl:element name="as:comment-import" xml:space="preserve">

                                    NIEM: subset domain schema referenced by exchange schema - $filenameStub
                                    NIEM: autogenerated from CAM template by XSLT
                                    NIEM: this schema is an import of type definitions for the main exchange schema
                                    NIEM: Namespace definitions for: $ns
                    </xsl:element>
                    <as:map-ns prefix="ansi-nist" type="Subset">Schemas/Subset/niem/ansi-nist/2.0</as:map-ns>
                    <as:map-ns prefix="apco" type="Subset">Schemas/Subset/niem/domains/apco/2.0</as:map-ns>
                    <as:map-ns prefix="atf" type="Subset">Schemas/Subset/niem/domains/atf/2.0</as:map-ns>
                    <as:map-ns prefix="can" type="Subset">Schemas/Subset/niem/domains/post-canada/2.0</as:map-ns>
                    <as:map-ns prefix="cap" type="Subset">Schemas/Subset/niem/domains/emergency/2.0</as:map-ns>
                    <as:map-ns prefix="cbrn" type="Subset">Schemas/Subset/niem/domains/cbrn/2.0</as:map-ns>
                    <as:map-ns prefix="cbrncl" type="Subset">Schemas/Subset/niem/domains/cbrncl/2.0</as:map-ns>
                    <as:map-ns prefix="census" type="Subset">Schemas/Subset/niem/domains/census/2.0</as:map-ns>
                    <as:map-ns prefix="de" type="Subset">Schemas/Subset/niem/domains/emergency/2.0</as:map-ns>
                    <as:map-ns prefix="dea" type="Subset">Schemas/Subset/niem/domains/dea/2.0</as:map-ns>
                    <as:map-ns prefix="edxl" type="Subset">Schemas/Subset/niem/domains/emergency/2.0</as:map-ns>
                    <as:map-ns prefix="edxl-cap" type="Subset">Schemas/Subset/niem/domains/emergency/2.0</as:map-ns>
                    <as:map-ns prefix="edxl-de" type="Subset">Schemas/Subset/niem/domains/emergency/2.0</as:map-ns>
                    <as:map-ns prefix="em" type="Subset">Schemas/Subset/niem/domains/emergency/2.0</as:map-ns>
                    <as:map-ns prefix="fbi" type="Subset">Schemas/Subset/niem/domains/fbi/2.0</as:map-ns>
                    <as:map-ns prefix="fs" type="Subset">Schemas/Subset/niem/domains/family/2.0</as:map-ns>
                    <as:map-ns prefix="geo" type="Subset">Schemas/Subset/niem/geo/2.0</as:map-ns>
                    <as:map-ns prefix="gml" type="Extension">Schemas/Extension/gml</as:map-ns>
                    <as:map-ns prefix="i" type="Subset">Schemas/Subset/niem/appinfo/2.0</as:map-ns>
                    <as:map-ns prefix="im" type="Subset">Schemas/Subset/niem/domains/domains/immigration/2.0</as:map-ns>
                    <as:map-ns prefix="intel" type="Subset">Schemas/Subset/niem/domains/intel/2.0</as:map-ns>
                    <as:map-ns prefix="iso_3166" type="Subset">Schemas/Subset/niem/iso_3166/2.0</as:map-ns>
                    <as:map-ns prefix="ip" type="Subset">Schemas/Subset/niem/domains/infrapro/2.0</as:map-ns>
                    <as:map-ns prefix="it" type="Subset">Schemas/Subset/niem/domains/trade/2.0</as:map-ns>
                    <as:map-ns prefix="j" type="Subset">Schemas/Subset/niem/domains/jxdm/4.0</as:map-ns>
                    <as:map-ns prefix="m" type="Subset">Schemas/Subset/niem/domains/maritime/2.0</as:map-ns>
                    <as:map-ns prefix="nc" type="Subset">Schemas/Subset/niem/niem-core</as:map-ns>
                    <as:map-ns prefix="niem-xsd" type="Subset">Schemas/Subset/niem/proxy/xsd/2.0</as:map-ns>
                    <as:map-ns prefix="om" type="Subset">Schemas/Subset/niem/domains/om/2.0</as:map-ns>
                    <as:map-ns prefix="s" type="Subset">Schemas/Subset/niem/structures</as:map-ns>
                    <as:map-ns prefix="sar" type="Subset">Schemas/Subset/niem/domains/sar/2.0</as:map-ns>
                    <as:map-ns prefix="scr" type="Subset">Schemas/Subset/niem/domains/scr/2.0</as:map-ns>

                    <as:map-ns URIcontains="constraint" type="Constraint">Schemas/Constraint</as:map-ns>
                    <as:map-ns URIcontains="extn" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns URIcontains="extension" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns URIcontains="niem" type="Extension">Schemas/Extension</as:map-ns>

                    <as:map-ns default="true" type="Exchange">Schemas/Exchange</as:map-ns>

                    <as:map-ns prefix="xlink" type="External">Schemas/External/xlink</as:map-ns>
                    <as:map-ns prefix="xpil" type="External">Schemas/External/ciq3.0</as:map-ns>
                    <as:map-ns prefix="axal" type="External">Schemas/External/ciq3.0</as:map-ns>
                    <as:map-ns prefix="nxnl" type="External">Schemas/External/ciq3.0</as:map-ns>
                    <as:map-ns prefix="gml" type="External">Schemas/External/gml</as:map-ns>
                    <as:map-ns prefix="look" type="External">Schemas/External/cam</as:map-ns>
                </as:xsd-mapping>
                <as:xsd-mapping type="OASIS Standard" date="03/19/2011">
                    <xsl:element name="as:comment-hdr" xml:space="preserve">

                       OASIS: Main schema - OASIS Standard (OS) specification schema
                       OASIS: Each namespace has its own import domain subset schema
                       OASIS: External Standard(s) schema are placed in the external folder
                    </xsl:element>
                    <xsl:element name="as:comment-end" xml:space="preserve">

                           OASIS: Copyright (C) OASIS Open 2001-2011. All Rights Reserved.
                           OASIS: Information on OASIS's  procedures with respect to rights in OASIS
                           OASIS: specifications can be found at the OASIS website.
                           OASIS: The terms and conditions of use may also be found at the OASIS website.
                           OASIS:  For complete legal license and use text see the OASIS website - http://www.oasis-open.org
                    </xsl:element>
                    <xsl:element name="as:comment-import" xml:space="preserve">

                    OASIS: Import schema - OASIS Standard specification schema
                    OASIS: This schema is an import of type definitions for the main schema
                    OASIS: Namespace definitions for: $ns
                    </xsl:element>
                    <!-- Note: matching is all made in lowercase -->
                    <as:map-ns default="true" type="Exchange">Schemas/OS</as:map-ns>
                    <as:map-ns URIcontains="extn" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns URIcontains="extension" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns prefix="xlink" type="External">Schemas/External/xlink</as:map-ns>
                    <as:map-ns prefix="gml" type="External">Schemas/External/gml</as:map-ns>
                    <as:map-ns prefix="ds" type="External">Schemas/External/dsig</as:map-ns>
                    <as:map-ns prefix="ts" type="External">Schemas/External/tsig</as:map-ns>
                    <as:map-ns prefix="gc" type="External">Schemas/OASIS/gcode</as:map-ns>
                    <as:map-ns prefix="xal" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="xnl" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="xnal" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="xpil" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="xprl" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="axal" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="nxnl" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="look" type="External">Schemas/OASIS/cam</as:map-ns>
                    <as:map-ns URIcontains="oasis" type="External">Schemas/OASIS</as:map-ns>
                    <as:map-ns URIcontains="www.w3.org" type="External">Schemas/W3C</as:map-ns>
                    <as:map-ns URIcontains="urn:" type="External">Schemas/External</as:map-ns>
                    <as:map-ns URIcontains="http:" type="External">Schemas/External</as:map-ns>
                </as:xsd-mapping>

                <as:xsd-mapping type="OASIS Draft" date="03/19/2011">
                    <xsl:element name="as:comment-hdr" xml:space="preserve">

                       OASIS: Main schema - OASIS Committee Specification (CS) schema
                       OASIS: Each namespace has its own import domain subset schema
                       OASIS: External Standard(s) schema are placed in the external folder
                    </xsl:element>
                    <xsl:element name="as:comment-end" xml:space="preserve">

                           OASIS: Copyright (C) OASIS Open 2001-2011. All Rights Reserved.
                           OASIS: Information on OASIS's  procedures with respect to rights in OASIS
                           OASIS: specifications can be found at the OASIS website.
                           OASIS: The terms and conditions of use may also be found at the OASIS website.
                           OASIS:  For complete legal license and use text see the OASIS website - http://www.oasis-open.org
                    </xsl:element>
                    <xsl:element name="as:comment-import" xml:space="preserve">

                    OASIS: Import schema - OASIS Committee Specification (CS) schema
                    OASIS: This schema is an import of type definitions for the main schema
                    OASIS: Namespace definitions for: $ns
                    </xsl:element>
                    <!-- Note: matching is all made in lowercase -->
                    <as:map-ns default="true" type="Exchange">Schemas/CS</as:map-ns>
                    <as:map-ns URIcontains="extn" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns URIcontains="extension" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns prefix="xlink" type="External">Schemas/External/xlink</as:map-ns>
                    <as:map-ns prefix="gml" type="External">Schemas/External/gml</as:map-ns>
                    <as:map-ns prefix="ds" type="External">Schemas/External/dsig</as:map-ns>
                    <as:map-ns prefix="ts" type="External">Schemas/External/tsig</as:map-ns>
                    <as:map-ns prefix="gc" type="External">Schemas/OASIS/gcode</as:map-ns>
                    <as:map-ns prefix="xal" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="xnl" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="xnal" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="xpil" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="xprl" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="axal" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="nxnl" type="External">Schemas/OASIS/ciq</as:map-ns>
                    <as:map-ns prefix="look" type="External">Schemas/OASIS/cam</as:map-ns>
                    <as:map-ns URIcontains="oasis" type="External">Schemas/OASIS</as:map-ns>
                    <as:map-ns URIcontains="www.w3.org" type="External">Schemas/W3C</as:map-ns>
                    <as:map-ns URIcontains="urn:" type="External">Schemas/External</as:map-ns>
                    <as:map-ns URIcontains="http:" type="External">Schemas/External</as:map-ns>
                </as:xsd-mapping>

                <as:xsd-mapping type="Other" date="01/06/2011">
                    <xsl:element name="as:comment-hdr" xml:space="preserve">

                    NOTICE: Generated from CAM template by XSLT
                    NOTICE: This schema is designed to maximize software interoperability and uses basic XSD Schema constructs only.
                    </xsl:element>
                    <xsl:element name="as:comment-end" xml:space="preserve">
                    NOTICE: Please refer to the original source owner schema, dictionary, and/or CAM template for updates and changes.
                    NOTICE: For copyright, licensing and use criteria please refer to owner authority.
                    </xsl:element>
                    <xsl:element name="as:comment-import" xml:space="preserve">

                    NOTICE: Generated from CAM template by XSLT
                    NOTICE: This schema is an import of type definitions for the main schema
                    NOTICE: Namespace definitions for: $ns
                    NOTICE: For copyright, licensing and use criteria please refer to owner authority.
                    </xsl:element>
                    <as:map-ns URIcontains="constraint" type="Constraint">Schemas/Constraint</as:map-ns>
                    <as:map-ns URIcontains="extn" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns URIcontains="extension" type="Extension">Schemas/Extension</as:map-ns>
                    <as:map-ns default="true" type="Exchange">Schemas/Exchange</as:map-ns>
                </as:xsd-mapping>
                <as:xsd-mapping type="default" date="01/06/2011">
                    <xsl:element name="as:comment-hdr" xml:space="preserve">

                    NOTICE: Generated from CAM template by XSLT
                    NOTICE:  This schema is designed to maximize software interoperability and uses basic XSD Schema constructs only.
                    </xsl:element>
                    <xsl:element name="as:comment-end" xml:space="preserve">
                    NOTICE: Please refer to the original source owner schema, dictionary, and/or CAM template for updates and changes.
                    NOTICE: For copyright, licensing and use criteria please refer to owner authority.
                    </xsl:element>
                    <xsl:element name="as:comment-import" xml:space="preserve">

                    NOTICE: Generated from CAM template by XSLT
                    NOTICE: This schema is an import of type definitions for the main schema
                    NOTICE: Namespace definitions for: $ns
                    NOTICE: For copyright, licensing and use criteria please refer to owner authority.
                    </xsl:element>
                    <as:map-ns default="true" />
                </as:xsd-mapping>

            </as:xsd-mappings>
        </xsl:variable>
        <xsl:copy-of select="$mappings"/>
    </xsl:function>

</xsl:stylesheet>
