<?xml version="1.0" encoding="UTF-8"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:lexs="http://usdoj.gov/leisp/lexs/3.1" 
    xmlns:w="http://niem.gov/niem/wantlist/1" 
    exclude-result-prefixes="xs xsd" version="2.0">

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: David RR Webber, April, 2008 -->

    <!-- 

Purpose : Merges two wantlists together - larger with smaller 
Original : 04/22/2008 
Initials : drrw 
CAM ver : 1.1 
Description : Part of a set of tools to ease alignment of XSD based validations and documentation 
and CAM templates. 
Updated - Inits - Change 
04/22/08 drrw Initial release 
    -->
    <xsl:param name="wantlistURL">EDXL-part-wantlist.xml</xsl:param>
   
    <xsl:output indent="yes"/>
    
    <xsl:variable name="wanted" select="document($wantlistURL)"/>

    <xsl:template match="/">

        <xsl:message><xsl:value-of select="concat('merge wantlist rules=',count($wanted/w:WantList/descendant::*))"></xsl:value-of></xsl:message>
        
        <xsl:variable name="ns-prefix">
            <xsl:value-of select="./@targetNamespace"/>
        </xsl:variable>              
            
            <xsl:for-each select="*">
                <!-- parses the entire XML input -->
                <xsl:copy>
                    <xsl:copy-of select=".//namespace::*"/>
                    <xsl:copy-of select="./@*"/>
                    <xsl:apply-templates mode="map" select="*">
                        <xsl:with-param name="fill" as="xsd:boolean" select="false()"/>
                    </xsl:apply-templates>                  
                </xsl:copy>
            </xsl:for-each>

    </xsl:template>
    
    <xsl:template match="Group" mode="map">
        <xsl:param name="fill" as="xsd:boolean"/>
        <xsl:variable name="this" select="@w:n"/>
        <xsl:choose>
            <xsl:when test="$fill or(@w:isXclude='1')">
                <xsl:message>Group: <xsl:value-of select="@w:n"/></xsl:message>
                <xsl:copy>
                    <xsl:attribute name="w:n" select="@w:n"/>
                    <xsl:attribute name="w:isXclude" select="'1'"/>
                    <xsl:apply-templates select="*" mode="map">
                        <xsl:with-param name="fill" select="true()"/>
                    </xsl:apply-templates>
                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="fillthis" as="xsd:boolean">
                    <xsl:choose>
                        <xsl:when test="$wanted/w:WantList//Group[@w:n=$this]"><xsl:value-of select="$wanted/w:WantList//Group[@w:n=$this]/@w:isXclude"/></xsl:when>
                        <xsl:otherwise>false</xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:copy>
                    <xsl:attribute name="w:n" select="@w:n"/>
                    <xsl:attribute name="w:isXclude" select="if($fillthis) then '1' else '0'"/>
                    <xsl:apply-templates select="*" mode="map">
                        <xsl:with-param name="fill" select="$fillthis"/>
                    </xsl:apply-templates>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="Element" mode="map">
        <xsl:param name="fill" as="xsd:boolean"/>
        <xsl:variable name="this" select="@w:n"/>
        <xsl:choose>
            <xsl:when test="$fill or(@w:isXclude='1')">
                <xsl:message>Element: <xsl:value-of select="@w:n"/></xsl:message>
                <xsl:copy>
                    <xsl:attribute name="w:n" select="@w:n"/>
                    <xsl:attribute name="w:isXclude" select="'1'"/>
                    <xsl:apply-templates select="*" mode="map">
                        <xsl:with-param name="fill" select="true()"/>
                    </xsl:apply-templates>
                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="fillthis" as="xsd:boolean">
                    <xsl:choose>
                        <xsl:when test="$wanted/w:WantList//Element[@w:n=$this]"><xsl:value-of select="$wanted/w:WantList//Element[@w:n=$this]/@w:isXclude"/></xsl:when>
                        <xsl:otherwise>false</xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:copy>
                    <xsl:attribute name="w:n" select="@w:n"/>
                    <xsl:attribute name="w:isXclude" select="if($fillthis) then '1' else '0'"/>
                    <xsl:apply-templates select="*" mode="map">
                        <xsl:with-param name="fill" select="$fillthis"/>
                    </xsl:apply-templates>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="Attribute" mode="map">
        <xsl:param name="fill" as="xsd:boolean"/>
        <xsl:variable name="this" select="@w:n"/>
        <xsl:choose>
            <xsl:when test="$fill or(@w:isXclude='1')">
                <xsl:copy>
                    <xsl:attribute name="w:n" select="@w:n"/>
                    <xsl:attribute name="w:isXclude" select="'1'"/>
                </xsl:copy>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="fillthis" as="xsd:boolean">
                    <xsl:choose>
                        <xsl:when test="$wanted/w:WantList//Attribute[@w:n=$this]"><xsl:value-of select="$wanted/w:WantList//Attribute[@w:n=$this]/@w:isXclude"/></xsl:when>
                        <xsl:otherwise>false</xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:copy>
                    <xsl:attribute name="w:n" select="@w:n"/>
                    <xsl:attribute name="w:isXclude" select="if($fillthis) then '1' else '0'"/>
                </xsl:copy>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

</xsl:stylesheet>
