<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:look="http://jcam.org.uk/LookupLists" xmlns:uuid="java:java.util.UUID"
    xmlns:xmi="http://schema.omg.org/spec/XMI/2.1" xmlns:uml="http://schema.omg.org/spec/UML/2.1.2"
    xmlns:as-xsl="http://www.oasis-open.org/committees/cam/xslt"
    xmlns:UML="org.omg.xmi.namespace.UML"
    exclude-result-prefixes="xs xsd xd uuid xmi uml UML as as-xsl look camed" version="2.0">
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> March 4, 2011</xd:p>
            <xd:p><xd:b>Author:</xd:b> DRRW</xd:p>
            <xd:p/>
        </xd:desc>
    </xd:doc>

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
            Author: David RR Webber, March, 2011 -->
    <!--
            Purpose  : Read in Dictionary file(s) - convert into XML for model viewer tool; return details back to Java via stdout. 
            Original : 03/27/2011
            Initials : drrw
            CAM ver    : 1.1 
            Description : Part of a set of tools to provide open source tools for using OASIS CAM dictionaries.
            Updated  - Inits - Change
            03/27/11   drrw    Initial release
            03/30/11   drrw    Align control file handling with Java
            03/30/11   drrw    Add selection of parent / itempath / item
            06/17/11   drrw    Implement compiled dictionary format to Mindmap
            06/20/11   drrw    Implement compiled dictionary format to XMI
            06/21/11   drrw    Implement compiled dictionary format to ArgoUML
            03/06/12   drrw    Apply same changes from model for UML model output to XMI
      
        -->

    <xsl:param name="locationFilePath">file:/temp/test/</xsl:param>
    <xsl:param name="controlFilePath">file:/temp/test/control-file.xml</xsl:param>
    <xsl:param name="modelType">MindMap</xsl:param>
    <xsl:param name="details">none, content, cardinality</xsl:param>
    <!-- none, cardinality, content, description -->

    <xsl:variable name="xslt-version">1.05</xsl:variable>
    <xsl:variable name="debug">0</xsl:variable>

    <xsl:variable name="dictionaryControlFile" select="document($controlFilePath)"/>

    <xsl:variable name="chkControlFile" select="count($dictionaryControlFile//camed:dictionary)"/>

    <xsl:variable name="stripCardinality"
        select="$dictionaryControlFile//camed:dictionary[@stripCardinality='true']/@shortname"/>

    <xsl:variable name="EpochSeconds">
        <!-- xsl:sequence select="current-dateTime()"/ -->
        <xsl:sequence
            select="( current-dateTime() - xs:dateTime('1970-01-01T00:00:00') )
            div
            xs:dayTimeDuration('PT1S')
            * 1000
            "
        />
    </xsl:variable>

    <xsl:variable name="number_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="text_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="token_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="NMToken_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="ID_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="IDREF_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="date_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="datetime_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="time_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="duration_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="decimal_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="double_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="float_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="integer_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="byte_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="boolean_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="long_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="short_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="list_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="NCName_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="QName_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="any_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="uri_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="language_uuid" select="uuid:randomUUID()"/>
    <xsl:variable name="binary_uuid" select="uuid:randomUUID()"/>

    <xsl:variable name="ArgoUML-documentation">
        <UML:TaggedValue.type>
            <UML:TagDefinition
                href="http://argouml.org/profiles/uml14/default-uml14.xmi#.:000000000000087C"/>
        </UML:TaggedValue.type>
    </xsl:variable>
    <xsl:variable name="ArgoUML-usage">
        <UML:TaggedValue.type>
            <UML:TagDefinition
                href="http://argouml.org/profiles/uml14/default-uml14.xmi#-64--88-0-101--2259be85:11dd526880c:-8000:000000000000E4B3"
            />
        </UML:TaggedValue.type>
    </xsl:variable>
    <xsl:variable name="ArgoUML-semantics">
        <UML:TaggedValue.type>
            <UML:TagDefinition
                href="http://argouml.org/profiles/uml14/default-uml14.xmi#-64--88-0-101--2259be85:11dd526880c:-8000:000000000000E4B0"
            />
        </UML:TaggedValue.type>
    </xsl:variable>

    <xsl:variable name="dictionary">
        <xsl:for-each select="$dictionaryControlFile//camed:dictionary">
            <xsl:variable name="thisFile" select="@location"/>
            <xsl:variable name="fileNamePath">
                <xsl:choose>
                    <xsl:when test="string-length($locationFilePath) = 0">
                        <xsl:value-of select="@location"/>
                    </xsl:when>
                    <xsl:when test="contains($locationFilePath,'file:/')">
                        <xsl:choose>
                            <xsl:when test="contains(@location,'file:/')">
                                <xsl:value-of select="@location"/>
                            </xsl:when>
                            <xsl:when test="starts-with(@location,'../')">
                                <xsl:value-of select="@location"/>
                            </xsl:when>
                            <!-- compute offset path relative to locationFilePath -->
                            <xsl:otherwise>
                                <xsl:value-of
                                    select="replace(concat($locationFilePath,@location),'//','/')"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="@location"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <components shortname="{@shortname}" namespaces="{@namespaces}">
                <!-- xsl:copy-of select="document($thisFile)"/ -->
                <xsl:variable name="tempDictionary" select="document($fileNamePath)"/>
                <ITEMSlookup>
                    <xsl:comment><xsl:value-of select="$tempDictionary/ITEMSlookup/comment()"/></xsl:comment>
                    <!-- now unpack the context -->
                    <xsl:for-each select="$tempDictionary//Property">
                        <xsl:variable name="propertyDetails" select="."/>
                        <xsl:variable name="uniqueContext" select="tokenize(./@context,'&#xA;')"/>
                        <xsl:for-each select="$uniqueContext">
                            <xsl:if test="string-length(.) &gt; 0">
                                <xsl:element name="Property">
                                    <xsl:for-each select="$propertyDetails/@*">
                                        <xsl:choose>
                                            <xsl:when test="name()='context'"/>
                                            <!-- skip existing context -->
                                            <xsl:when test="name()='group'"/>
                                            <!-- skip  -->
                                            <xsl:when test="name()='where'"/>
                                            <!-- skip  -->
                                            <xsl:when test="name()='notes'"/>
                                            <!-- skip  -->
                                            <xsl:when test="name()='language'"/>
                                            <!-- skip  -->
                                            <xsl:when test="name()='anno'"/>
                                            <!-- skip  -->
                                            <xsl:otherwise>
                                                <xsl:attribute name="{name()}" select="."/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:for-each>
                                    <xsl:attribute name="context" select="."/>
                                </xsl:element>
                            </xsl:if>
                        </xsl:for-each>
                    </xsl:for-each>
                </ITEMSlookup>
            </components>
        </xsl:for-each>
    </xsl:variable>

    <xsl:variable name="fullDictionary">
        <fullDictionary>
            <xsl:for-each select="$dictionaryControlFile//camed:collection">
                <xsl:for-each select="camed:dictionary">
                    <xsl:variable name="fileNamePath">
                        <xsl:choose>
                            <xsl:when test="string-length($locationFilePath) = 0">
                                <xsl:value-of select="@location"/>
                            </xsl:when>
                            <xsl:when test="contains($locationFilePath,'file:/')">
                                <xsl:choose>
                                    <xsl:when test="contains(@location,'file:/')">
                                        <xsl:value-of select="@location"/>
                                    </xsl:when>
                                    <xsl:when test="starts-with(@location,'../')">
                                        <xsl:value-of select="@location"/>
                                    </xsl:when>
                                    <!-- compute offset path relative to locationFilePath -->
                                    <xsl:otherwise>
                                        <xsl:value-of
                                            select="replace(concat($locationFilePath,@location),'//','/')"
                                        />
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="@location"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:message> - loading dictionary: <xsl:value-of select="$fileNamePath"
                        /></xsl:message>
                    <Collection>
                        <xsl:variable name="dictionaryFile" select="document($fileNamePath)"/>
                        <xsl:copy-of select="$dictionaryFile"/>
                    </Collection>
                    <xsl:variable name="shortname" select="@shortname"/>
                    <xsl:message> - processing dictionary: <xsl:value-of select="$fileNamePath"
                        /></xsl:message>
                </xsl:for-each>
            </xsl:for-each>
        </fullDictionary>
    </xsl:variable>

    <xsl:variable name="rules" select="$fullDictionary//Collection/Dictionary/Rules"/>
    <xsl:variable name="dictionaryDetails" select="$fullDictionary//Collection/Dictionary"/>
    <xsl:variable name="excludes">
        <empty/>
    </xsl:variable>
    <!-- compatibility with cam-2-model -->

    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"
        exclude-result-prefixes="xd xs xsd look as camed"/>

    <xsl:template match="/">
        <xsl:message>Dictionary to model generator starting <xsl:value-of
                select="dateTime(current-date(),current-time())"/></xsl:message>
        <xsl:message>Version: <xsl:value-of select="$xslt-version"/></xsl:message>
        <xsl:message>Model mode: <xsl:value-of select="$modelType"/></xsl:message>
        <xsl:message>Content selections: <xsl:value-of select="$details"/></xsl:message>
        <xsl:message>Control File Location: <xsl:value-of select="$locationFilePath"/></xsl:message>
        <xsl:variable name="loadDictionary">
            <xsl:value-of select="$fullDictionary//Collection/@nosuchattribute"/>
        </xsl:variable>
        <xsl:message>Dictionary Collections -</xsl:message>
        <xsl:value-of select="$loadDictionary"/>
        <xsl:message>Total Parent items: <xsl:value-of
                select="count($fullDictionary//Collection/Dictionary/Parent)"/></xsl:message>
        <xsl:if test="empty($dictionaryControlFile)">
            <xsl:message terminate="yes">ERROR: Dictionary control file missing!</xsl:message>
        </xsl:if>
        <xsl:if test="$chkControlFile &lt; 1">
            <xsl:message terminate="yes">ERROR: Dictionary control file is invalid!</xsl:message>
        </xsl:if>

        <xsl:choose>
            <xsl:when test="$modelType='MindMap'">
                <xsl:message>Writing model</xsl:message>
                <xsl:element name="map" exclude-result-prefixes="#all">
                    <xsl:attribute name="version">0.9.0</xsl:attribute>
                    <xsl:comment>To view this file, download free mind mapping software FreeMind from http://freemind.sourceforge.net
                        Model autogenerated by Dictionary-Model
                        Script: v<xsl:value-of select="$xslt-version"/>
                        Selections: <xsl:value-of select="$details"/>
                    </xsl:comment>
                    <attribute_registry SHOW_ATTRIBUTES="hide"/>

                    <xsl:for-each select="$dictionaryControlFile//camed:collection">
                        <node BACKGROUND_COLOR="#ff6666" CREATED="{$EpochSeconds}"
                            ID="ID_{generate-id($dictionaryControlFile//camed:collection[1]/@name)}"
                            MODIFIED="{$EpochSeconds}"
                            TEXT="{$dictionaryControlFile//camed:collection[1]/@name}">
                            <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                            <xsl:for-each select="camed:dictionary">
                                <node BACKGROUND_COLOR="#ff6666" CREATED="{$EpochSeconds}"
                                    ID="ID_{generate-id($dictionaryControlFile//camed:collection[1]/@name)}"
                                    MODIFIED="{$EpochSeconds}" STYLE="bubble" FOLDED="true"
                                    TEXT="{@name}">
                                    <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                    <xsl:variable name="fileNamePath">
                                        <xsl:choose>
                                            <xsl:when test="string-length($locationFilePath) = 0">
                                                <xsl:value-of select="@location"/>
                                            </xsl:when>
                                            <xsl:when test="contains($locationFilePath,'file:/')">
                                                <xsl:choose>
                                                  <xsl:when test="contains(@location,'file:/')">
                                                  <xsl:value-of select="@location"/>
                                                  </xsl:when>
                                                  <xsl:when test="starts-with(@location,'../')">
                                                  <xsl:value-of select="@location"/>
                                                  </xsl:when>
                                                  <!-- compute offset path relative to locationFilePath -->
                                                  <xsl:otherwise>
                                                  <xsl:value-of
                                                  select="replace(concat($locationFilePath,@location),'//','/')"
                                                  />
                                                  </xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:value-of select="@location"/>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:variable>

                                    <xsl:variable name="dictionaryFile"
                                        select="document($fileNamePath)"/>
                                    <xsl:variable name="shortname" select="@shortname"/>
                                    <xsl:message> - processing dictionary: <xsl:value-of
                                            select="$fileNamePath"/></xsl:message>
                                    <xsl:choose>
                                        <xsl:when test="$dictionaryFile//IEPD-Factory">
                                            <!-- Place holder here for future model input extensions -->
                                            <xsl:element name="Dictionary">
                                                <xsl:attribute name="name" select="@name"/>
                                                <xsl:attribute name="description"
                                                  select="camed:annotation/camed:documentation"/>
                                                <xsl:attribute name="item"
                                                  select="camed:annotation/@item"/>
                                                <xsl:attribute name="shortname" select="@shortname"/>
                                                <xsl:attribute name="namespaces"
                                                  select="@namespaces"/>
                                                <xsl:apply-templates mode="IEPD-Factory"
                                                  select="$dictionaryFile"/>
                                            </xsl:element>
                                        </xsl:when>
                                        <xsl:when test="$dictionaryFile//Dictionary">
                                            <!-- Process the compiled format of the dictionary -->
                                            <xsl:comment>Dictionary details: <xsl:value-of select="$dictionaryFile/ITEMSlookup/comment()"/></xsl:comment>
                                            <xsl:for-each
                                                select="$dictionaryFile//Dictionary/Parent">
                                                <xsl:if test="not(@name = 'moreComponents')">
                                                  <xsl:variable name="cardinality">
                                                  <xsl:if test="contains($details,'cardinality')">
                                                  <xsl:if test="@repeatable='true'">*</xsl:if>
                                                  <xsl:if test="@optional='true'">?</xsl:if>
                                                  </xsl:if>
                                                  </xsl:variable>
                                                  <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id(.)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="bubble" FOLDED="true"
                                                  TEXT="{concat,@name,' ',$cardinality}"
                                                  xsl:exclude-result-prefixes="#all">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                  <xsl:if test="contains($details,'content')">
                                                  <xsl:variable name="findUID" select="@UID"/>
                                                  <xsl:variable name="Rules"
                                                  select="$dictionaryFile//Rules/Rule[UID=$findUID]"/>
                                                  <xsl:for-each select="$Rules">
                                                  <xsl:if test="@type">
                                                  <attribute NAME="type" VALUE="{@type}"/>
                                                  </xsl:if>
                                                  <xsl:if test="@mask">
                                                  <attribute NAME="mask" VALUE="{@mask}"/>
                                                  </xsl:if>
                                                  </xsl:for-each>
                                                  </xsl:if>
                                                  <xsl:apply-templates select="*" mode="compiled">
                                                  <xsl:with-param name="dictionaryFile"
                                                  select="$dictionaryFile"/>
                                                  </xsl:apply-templates>
                                                  </node>
                                                </xsl:if>
                                            </xsl:for-each>

                                            <xsl:if
                                                test="$dictionaryFile//Parent[@name='moreComponents']/Child">
                                                <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id(.)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  TEXT="More Components">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                  <richcontent TYPE="NOTE">
                                                  <html>
                                                  <head/>
                                                  <body>
                                                  <p>These are standalone atomic components and
                                                  types available in the dictionary.</p>
                                                  </body>
                                                  </html>
                                                  </richcontent>
                                                  <xsl:for-each
                                                  select="$dictionaryFile//Parent[@name='moreComponents']/Child">
                                                  <xsl:variable name="cardinality">
                                                  <xsl:if test="contains($details,'cardinality')">
                                                  <xsl:if test="@repeatable='true'">*</xsl:if>
                                                  <xsl:if test="@optional='true'">?</xsl:if>
                                                  </xsl:if>
                                                  </xsl:variable>
                                                  <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id(.)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="bubble" FOLDED="true"
                                                  TEXT="{concat,@name,' ',$cardinality}"
                                                  xsl:exclude-result-prefixes="#all">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                  <xsl:if test="contains($details,'content')">
                                                  <xsl:variable name="findUID" select="@UID"/>
                                                  <xsl:variable name="Rules"
                                                  select="$dictionaryFile//Rules/Rule[UID=$findUID]"/>
                                                  <xsl:for-each select="$Rules">
                                                  <xsl:if test="@type">
                                                  <attribute NAME="type" VALUE="{@type}"/>
                                                  </xsl:if>
                                                  <xsl:if test="@mask">
                                                  <attribute NAME="mask" VALUE="{@mask}"/>
                                                  </xsl:if>
                                                  </xsl:for-each>
                                                  </xsl:if>
                                                  <!-- node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}" MODIFIED="{$EpochSeconds}" POSITION="right" STYLE="fork" TEXT="{concat(@name,' ',$cardinality)}">
                                                    <font BOLD="false" NAME="SansSerif" SIZE="12"/>
                                                    <xsl:if test="$content/attribute and(contains($details,'content'))"><xsl:copy-of select="$content"/></xsl:if>
                                                    <xsl:if test="string-length(@definition) &gt; 0 and(contains($details,'description'))"><richcontent TYPE="NOTE"><html><head/><body><p><xsl:value-of select="@definition"/></p></body></html></richcontent></xsl:if>
                                                </node -->
                                                  <xsl:for-each select="Attribute">
                                                  <!-- Attribute -->
                                                  <xsl:variable name="cardinality">
                                                  <xsl:if test="contains($details,'cardinality')">
                                                  <xsl:if test="@repeatable='true'">*</xsl:if>
                                                  <xsl:if test="@optional='true'">?</xsl:if>
                                                  </xsl:if>
                                                  </xsl:variable>
                                                  <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id(.)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="fork"
                                                  TEXT="@{concat,@name,' ',$cardinality}"
                                                  xsl:exclude-result-prefixes="#all">
                                                  <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                                                  <xsl:variable name="findUID" select="@UID"/>
                                                  <xsl:variable name="RulesAttr"
                                                  select="$dictionaryFile//Rules/Rule[UID=$findUID]"/>
                                                  <xsl:variable name="content">
                                                  <xsl:if test="contains($details,'content')">
                                                  <xsl:for-each select="$RulesAttr">
                                                  <xsl:if test="@type">
                                                  <attribute NAME="type" VALUE="{@type}"/>
                                                  </xsl:if>
                                                  <xsl:if test="@mask">
                                                  <attribute NAME="mask" VALUE="{@mask}"/>
                                                  </xsl:if>
                                                  </xsl:for-each>
                                                  </xsl:if>
                                                  </xsl:variable>
                                                  <xsl:if
                                                  test="$content/attribute and(contains($details,'content'))">
                                                  <xsl:copy-of select="$content"/>
                                                  </xsl:if>
                                                  <xsl:if
                                                  test="string-length(@definition) &gt; 0 and(contains($details,'description'))">
                                                  <richcontent TYPE="NOTE">
                                                  <html>
                                                  <head/>
                                                  <body>
                                                  <p>
                                                  <xsl:value-of select="@definition"/>
                                                  </p>
                                                  </body>
                                                  </html>
                                                  </richcontent>
                                                  </xsl:if>
                                                  </node>
                                                  </xsl:for-each>
                                                  </node>
                                                  </xsl:for-each>
                                                </node>
                                            </xsl:if>
                                        </xsl:when>
                                        <!-- debugging aid to process only first entry in dictionary list -->
                                        <!-- xsl:when test="$debug = 0 and(position() = 2)"/>
                                    <xsl:when test="position() &gt; 2"/ -->
                                        <xsl:otherwise>

                                            <xsl:comment>Dictionary details: <xsl:value-of select="$dictionaryFile/ITEMSlookup/comment()"/></xsl:comment>

                                            <xsl:if test="$stripCardinality[. = $shortname]">
                                                <xsl:message> Strip Cardinality for <xsl:value-of
                                                  select="$shortname"/> = true</xsl:message>
                                            </xsl:if>

                                            <xsl:for-each
                                                select="$dictionaryFile//Property[@CCTS='ABIE']">
                                                <xsl:if
                                                  test="normalize-space(@context) = concat('/',@item)">
                                                  <!-- Pick parents off the root - not within other parents -->
                                                  <xsl:variable name="ABIEdetails">
                                                  <xsl:choose>
                                                  <xsl:when test="ends-with(@item,'Type')">
                                                  <!-- If its name ends in Type then rename it -->
                                                  <xsl:variable name="oldName" select="@item"/>
                                                  <xsl:variable name="newName"
                                                  select="replace(concat(@item,'ZZZ'),'TypeZZZ','Details')"/>
                                                  <xsl:element name="Property">
                                                  <xsl:for-each select="@*">
                                                  <xsl:choose>
                                                  <xsl:when test="name()='item'">
                                                  <xsl:attribute name="item" select="$newName"/>
                                                  </xsl:when>
                                                  <xsl:when test="name()='children'">
                                                  <xsl:attribute name="children"
                                                  select="replace(replace(.,concat('/',$oldName,'&#xA;'),concat('/',$newName,'&#xA;')),concat(';',$oldName,'&#xA;'),concat(';',$newName,'&#xA;'))"
                                                  />
                                                  </xsl:when>
                                                  <xsl:when test="name()='context'">
                                                  <xsl:attribute name="context"
                                                  select="concat('/',$newName)"/>
                                                  </xsl:when>
                                                  <xsl:when test="name()='label'">
                                                  <xsl:variable name="labelText"
                                                  select="substring-before(concat(normalize-space(.),'ZZZ'),'TypeZZZ')"/>
                                                  <xsl:attribute name="label"
                                                  select="concat($labelText,'Details')"/>
                                                  </xsl:when>
                                                  <xsl:otherwise>
                                                  <xsl:copy-of select="."/>
                                                  </xsl:otherwise>
                                                  </xsl:choose>
                                                  </xsl:for-each>
                                                  </xsl:element>
                                                  </xsl:when>
                                                  <xsl:otherwise>
                                                  <xsl:copy-of select="."/>
                                                  </xsl:otherwise>
                                                  </xsl:choose>
                                                  </xsl:variable>

                                                  <xsl:apply-templates mode="CAM"
                                                  select="$ABIEdetails">
                                                  <xsl:with-param name="shortname"
                                                  select="$shortname"/>
                                                  <xsl:with-param name="dictionary"
                                                  select="$dictionaryFile"/>
                                                  <xsl:with-param name="itemPath" select="'|'"/>
                                                  <xsl:with-param name="parent"/>
                                                  </xsl:apply-templates>
                                                </xsl:if>
                                            </xsl:for-each>

                                            <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}"
                                                MODIFIED="{$EpochSeconds}" POSITION="right"
                                                TEXT="More Components">
                                                <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                <richcontent TYPE="NOTE">
                                                  <html>
                                                  <head/>
                                                  <body>
                                                  <p>These are standalone atomic components and
                                                  types available in the dictionary.</p>
                                                  </body>
                                                  </html>
                                                </richcontent>
                                                <xsl:for-each
                                                  select="$dictionaryFile//Property[@CCTS='BBIE']">
                                                  <xsl:if test="@context = concat('/',@item)">
                                                  <xsl:variable name="cardinality">
                                                  <xsl:if test="contains(@cardinality,'repeatable')"
                                                  >*</xsl:if>
                                                  <xsl:if test="contains(@cardinality,'optional')"
                                                  >?</xsl:if>
                                                  </xsl:variable>
                                                  <xsl:variable name="content">
                                                  <xsl:if test="@mask &gt; ''">
                                                  <attribute NAME="{@type}" VALUE="{@mask}"/>
                                                  </xsl:if>
                                                  </xsl:variable>
                                                  <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id(.)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="fork"
                                                  TEXT="{concat(@item,' ',$cardinality)}">
                                                  <font BOLD="false" NAME="SansSerif" SIZE="12"/>
                                                  <xsl:if
                                                  test="$content/attribute and(contains($details,'content'))">
                                                  <xsl:copy-of select="$content"/>
                                                  </xsl:if>
                                                  <xsl:if
                                                  test="string-length(@definition) &gt; 0 and(contains($details,'description'))">
                                                  <richcontent TYPE="NOTE">
                                                  <html>
                                                  <head/>
                                                  <body>
                                                  <p>
                                                  <xsl:value-of select="@definition"/>
                                                  </p>
                                                  </body>
                                                  </html>
                                                  </richcontent>
                                                  </xsl:if>
                                                  </node>
                                                  </xsl:if>
                                                </xsl:for-each>
                                            </node>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </node>
                            </xsl:for-each>
                        </node>
                    </xsl:for-each>

                    <!-- xsl:copy-of select="$rules"/ -->
                    <!-- xsl:copy-of select="$descriptions"/ -->
                </xsl:element>
                <xsl:message>Model completed</xsl:message>
            </xsl:when>
            <xsl:when test="$modelType='UML model'">
                <xsl:message>Writing dictionary XMI model</xsl:message>
                <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                <xsl:variable name="dictionaryCollection"
                    select="$dictionaryControlFile//camed:collection/@name"/>
                <xsl:variable name="umlComment"
                    select="concat('Dictionary created: ',current-dateTime(),'&#10; Comment: ',$dictionaryCollection,'&#10;')"/>
                <xsl:element name="xmi:XMI">
                    <xsl:namespace name="uml">http://schema.omg.org/spec/UML/2.1.2</xsl:namespace>
                    <xsl:namespace name="xmi">http://schema.omg.org/spec/XMI/2.1</xsl:namespace>
                    <xsl:attribute name="xmi:version">2.1</xsl:attribute>
                    <xsl:variable name="dictionaryDescription"
                        select="$dictionaryDetails/.[1]/@description"/>
                    <xsl:variable name="dictionaryName" select="$dictionaryDetails/.[1]/@name"/>

                    <xsl:comment>To view this file, please use a modelling tool compatible with XMI v2.1 syntax.
                        Model autogenerated by CAM-2-model
                        Description: <xsl:value-of select="$dictionaryDescription[1]"/> 
                        Version: <xsl:value-of select="$dictionaryName[1]"/> 
                        Script: v<xsl:value-of select="$xslt-version"/>
                        Selections: <xsl:value-of select="$details"/>
                    </xsl:comment>
                    <xmi:Documentation exporter="CAMeditor" exporterVersion="{$xslt-version}"/>
                    <uml:Model xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="{$dictionaryCollection}"
                        visibility="public">
                        <ownedComment xmi:type="uml:Comment" xmi:id="{uuid:randomUUID()}"
                            body="{$umlComment}">
                            <annotatedElement xmi:idref="{$uuid}"/>
                        </ownedComment>
                        <xsl:variable name="root-uuid" select="uuid:randomUUID()"/>
                        <packagedElement xmi:type="uml:Package" xmi:id="{$root-uuid}"
                            xmi:uuid="{$root-uuid}" name="Components">
                            <xsl:apply-templates
                                select="$fullDictionary//Collection/Dictionary/Parent" mode="XMI"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$text_uuid}"
                                xmi:uuid="{$text_uuid}" name="TextType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$number_uuid}"
                                xmi:uuid="{$number_uuid}" name="NumberType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$token_uuid}"
                                xmi:uuid="{$token_uuid}" name="TokenType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$NMToken_uuid}"
                                xmi:uuid="{$NMToken_uuid}" name="NMTokenType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$ID_uuid}"
                                xmi:uuid="{$ID_uuid}" name="IDType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$IDREF_uuid}"
                                xmi:uuid="{$IDREF_uuid}" name="IDRefType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$date_uuid}"
                                xmi:uuid="{$date_uuid}" name="DateType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$datetime_uuid}"
                                xmi:uuid="{$datetime_uuid}" name="DateTimeType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$time_uuid}"
                                xmi:uuid="{$time_uuid}" name="TimeType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$duration_uuid}"
                                xmi:uuid="{$duration_uuid}" name="DurationType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$decimal_uuid}"
                                xmi:uuid="{$decimal_uuid}" name="DecimalType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$double_uuid}"
                                xmi:uuid="{$double_uuid}" name="DoubleType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$float_uuid}"
                                xmi:uuid="{$float_uuid}" name="FloatType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$integer_uuid}"
                                xmi:uuid="{$integer_uuid}" name="IntegerType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$byte_uuid}"
                                xmi:uuid="{$byte_uuid}" name="ByteType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$boolean_uuid}"
                                xmi:uuid="{$boolean_uuid}" name="BooleanType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$long_uuid}"
                                xmi:uuid="{$long_uuid}" name="LongType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$short_uuid}"
                                xmi:uuid="{$short_uuid}" name="ShortType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$list_uuid}"
                                xmi:uuid="{$list_uuid}" name="ListType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$NCName_uuid}"
                                xmi:uuid="{$NCName_uuid}" name="NCNameType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$QName_uuid}"
                                xmi:uuid="{$QName_uuid}" name="QNameType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$any_uuid}"
                                xmi:uuid="{$any_uuid}" name="AnyType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$uri_uuid}"
                                xmi:uuid="{$uri_uuid}" name="URIType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$language_uuid}"
                                xmi:uuid="{$language_uuid}" name="LanguageType"/>
                            <packagedElement xmi:type="uml:Class" xmi:id="{$binary_uuid}"
                                xmi:uuid="{$binary_uuid}" name="BinaryType"/>
                        </packagedElement>
                    </uml:Model>
                </xsl:element>
                <xsl:message>Model completed</xsl:message>
            </xsl:when>
            <xsl:when test="$modelType='ArgoUML model'">
                <xsl:message>Writing ArgoUML model</xsl:message>
                <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                <xsl:variable name="dictionaryDescription"
                    select="$dictionaryDetails/.[1]/@description"/>
                <xsl:variable name="dictionaryName" select="$dictionaryDetails/.[1]/@name"/>
                <XMI xmi.version="1.2" xmlns:UML="org.omg.xmi.namespace.UML"
                    timestamp="{dateTime(current-date(),current-time())}">
                    <xsl:comment>To view this file, please use ArgoUML modelling tool.
                        Model autogenerated by CAM-2-model
                        Description: <xsl:value-of select="$dictionaryDescription[1]"/> 
                        Version: <xsl:value-of select="$dictionaryName[1]"/> 
                        Script: v<xsl:value-of select="$xslt-version"/>
                        Selections: <xsl:value-of select="$details"/>
                    </xsl:comment>
                    <XMI.header>
                        <XMI.documentation>
                            <XMI.exporter>ArgoUML from CAMeditor tool</XMI.exporter>
                            <XMI.exporterVersion><xsl:value-of select="$xslt-version"/> xslt
                                exporter CAM-2-model $</XMI.exporterVersion>
                        </XMI.documentation>
                        <XMI.metamodel xmi.name="UML" xmi.version="1.4"/>
                    </XMI.header>
                    <XMI.content>
                        <UML:Model xmi.id="{$uuid}" name="{$dictionaryName[1]}"
                            isSpecification="false" isRoot="false" isLeaf="false" isAbstract="false">
                            <xsl:apply-templates
                                select="$fullDictionary//Collection/Dictionary/Parent"
                                mode="ArgoUML"/>
                            <UML:Namespace.ownedElement>
                                <UML:Class xmi.id="{$text_uuid}" name="TextType" visibility="public"
                                    isSpecification="false" isRoot="false" isLeaf="false"
                                    isAbstract="false" isActive="false"> </UML:Class>
                            </UML:Namespace.ownedElement>
                        </UML:Model>
                    </XMI.content>
                </XMI>
                <xsl:message>Model completed</xsl:message>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message terminate="yes">ERROR: Unknown model output type: <xsl:value-of
                        select="$modelType"/></xsl:message>
            </xsl:otherwise>
        </xsl:choose>

        <xsl:message>Finished Dictionary model generation</xsl:message>
    </xsl:template>

    <xsl:template match="*" mode="IEPD-Factory">
        <xsl:comment>IEPD Factory dictionary mode</xsl:comment>
        <!-- Placeholder - no logic currently here for IEPD factory dictionaries -->
    </xsl:template>

    <xsl:template match="@*" mode="IEPD-Factory">
        <xsl:copy-of select="."/>
    </xsl:template>

    <xsl:template match="Property[@CCTS='ABIE']" mode="CAM">
        <xsl:param name="shortname"/>
        <xsl:param name="dictionary"/>
        <xsl:param name="itemPath"/>
        <xsl:param name="parent"/>

        <xsl:variable name="stripChildCardinality">
            <xsl:choose>
                <xsl:when test="$stripCardinality[. = $shortname]">true</xsl:when>
                <xsl:otherwise>false</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:choose>
            <xsl:when test="contains(@item,'AllRootLevelTypeDefinitions')"/>
            <!-- ignore root element -->
            <xsl:otherwise>
                <xsl:variable name="itself" select="."/>
                <!-- retain copy of itself for recursive checking -->
                <xsl:variable name="cardinality">
                    <xsl:if test="contains(@cardinality,'repeatable')">*</xsl:if>
                    <xsl:if test="contains(@cardinality,'optional')">?</xsl:if>
                </xsl:variable>
                <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}" MODIFIED="{$EpochSeconds}"
                    POSITION="right" STYLE="bubble" FOLDED="true"
                    TEXT="{concat,@item,' ',$cardinality}" xsl:exclude-result-prefixes="#all">
                    <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                    <xsl:variable name="content">
                        <xsl:if test="@mask &gt; ''">
                            <attribute NAME="{@type}" VALUE="{@mask}"/>
                        </xsl:if>
                    </xsl:variable>
                    <xsl:if test="$content/attribute and(contains($details,'content'))">
                        <xsl:copy-of select="$content"/>
                    </xsl:if>
                    <xsl:if
                        test="string-length(@definition) &gt; 0 and(contains($details,'description'))">
                        <richcontent TYPE="NOTE">
                            <html>
                                <head/>
                                <body>
                                    <p>
                                        <xsl:value-of select="@definition"/>
                                    </p>
                                </body>
                            </html>
                        </richcontent>
                    </xsl:if>

                    <xsl:variable name="thisParent"
                        select="if(ends-with(@item,'Details')) then replace(concat(@item,'ZZZ'),'DetailsZZZ','Type') else @item"/>
                    <xsl:variable name="childrenChk" select="substring-after(@children,@item)"/>
                    <xsl:for-each select="tokenize(@children,'&#xA;')">
                        <xsl:choose>
                            <xsl:when test="string-length(.) &lt; 1"/>
                            <!-- ignore empty token -->
                            <xsl:when test="$thisParent = .">
                                <xsl:if test="$debug=0">
                                    <xsl:message> - ignored recursive child <xsl:value-of select="."
                                        /></xsl:message>
                                </xsl:if>
                            </xsl:when>
                            <!-- ignore recursive references -->
                            <xsl:when test="position() &gt; 1">
                                <!-- process from after first token -->
                                <xsl:variable name="this" select="."/>
                                <xsl:variable name="context"
                                    select="replace(concat('/',$thisParent,'/',$this,'&#xA;'),'//','/')"/>
                                <xsl:variable name="moreContext"
                                    select="replace(concat('/',$thisParent,'/',$this),'//','/')"/>
                                <!--  -->
                                <xsl:variable name="singleContext"
                                    select="replace(concat('/',$thisParent,'/',$this),'//','/')"/>
                                <xsl:variable name="parentContext"
                                    select="replace(concat('/',$parent,$singleContext),'//','/')"/>
                                <xsl:variable name="fullContext"
                                    select="replace(concat($parent,'/',$this),'//','/')"/>
                                <xsl:variable name="itemContext"
                                    select="replace(concat('/',$parent,replace($itemPath,'\|','//'),$this),'//','/')"/>

                                <xsl:choose>
                                    <xsl:when test="$this = $thisParent">
                                        <!-- Recursive reference to parent -->
                                        <xsl:message> - recursive child of <xsl:value-of
                                                select="$thisParent"/></xsl:message>
                                        <node CREATED="{$EpochSeconds}"
                                            ID="ID_{generate-id($itself)}"
                                            MODIFIED="{$EpochSeconds}" POSITION="right"
                                            STYLE="bubble" TEXT="{concat($this,' RECURSIVE')}">
                                            <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                        </node>
                                    </xsl:when>
                                    <xsl:when
                                        test="starts-with($this,'@') and($dictionary//Property[@CCTS='ASBIE'][@item=substring-after($this,'@')])">
                                        <!-- Attribute -->
                                        <xsl:variable name="ASBIEdetails"
                                            select="$dictionary//Property[@CCTS='ASBIE'][@item=substring-after($this,'@')]"/>
                                        <node CREATED="{$EpochSeconds}"
                                            ID="ID_{generate-id($itself)}"
                                            MODIFIED="{$EpochSeconds}" POSITION="right" STYLE="fork"
                                            TEXT="@{substring-after($this,'@')}">
                                            <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                                            <xsl:variable name="content">
                                                <xsl:if test="$ASBIEdetails[1]/@mask &gt; ''">
                                                  <attribute NAME="{$ASBIEdetails[1]/@type}"
                                                  VALUE="{$ASBIEdetails[1]/@mask}"/>
                                                </xsl:if>
                                            </xsl:variable>
                                            <xsl:if
                                                test="$content/attribute and(contains($details,'content'))">
                                                <xsl:copy-of select="$content"/>
                                            </xsl:if>
                                            <xsl:if
                                                test="string-length($ASBIEdetails[1]/@definition) &gt; 0 and(contains($details,'description'))">
                                                <richcontent TYPE="NOTE">
                                                  <html>
                                                  <head/>
                                                  <body>
                                                  <p>
                                                  <xsl:value-of
                                                  select="$ASBIEdetails[1]/@definition"/>
                                                  </p>
                                                  </body>
                                                  </html>
                                                </richcontent>
                                            </xsl:if>
                                        </node>
                                    </xsl:when>
                                    <xsl:when
                                        test="$dictionary//Property[@CCTS='BBIE'][@item=$this]">
                                        <xsl:variable name="BBIEdetails"
                                            select="$dictionary//Property[@CCTS='BBIE'][@item=$this]"/>
                                        <node CREATED="{$EpochSeconds}"
                                            ID="ID_{generate-id($itself)}"
                                            MODIFIED="{$EpochSeconds}" POSITION="right" STYLE="fork"
                                            TEXT="{$this}">
                                            <font BOLD="false" NAME="SansSerif" SIZE="12"/>
                                            <xsl:variable name="content">
                                                <xsl:if test="$BBIEdetails[1]/@mask &gt; ''">
                                                  <attribute NAME="{$BBIEdetails[1]/@type}"
                                                  VALUE="{$BBIEdetails[1]/@mask}"/>
                                                </xsl:if>
                                            </xsl:variable>
                                            <xsl:if
                                                test="$content/attribute and(contains($details,'content'))">
                                                <xsl:copy-of select="$content"/>
                                            </xsl:if>
                                            <xsl:if
                                                test="string-length($BBIEdetails[1]/@definition) &gt; 0 and(contains($details,'description'))">
                                                <richcontent TYPE="NOTE">
                                                  <html>
                                                  <head/>
                                                  <body>
                                                  <p>
                                                  <xsl:value-of select="$BBIEdetails[1]/@definition"
                                                  />
                                                  </p>
                                                  </body>
                                                  </html>
                                                </richcontent>
                                            </xsl:if>
                                        </node>
                                    </xsl:when>
                                    <xsl:when
                                        test="$dictionary//Property[@CCTS='ABIE'][@item=$this][@context=$singleContext]">
                                        <xsl:variable name="newParent"
                                            select="$dictionary//Property[@CCTS='ABIE'][@item=$this][@context=$singleContext]"/>
                                        <xsl:choose>
                                            <xsl:when
                                                test="contains($itemPath,concat('|',$this,'|'))">
                                                <!-- Already in parent, so prevent looping -->
                                                <xsl:message> - recursive parent - <xsl:value-of
                                                  select="$this"/></xsl:message>
                                                <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id($itself)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="bubble" TEXT="{concat($this,' RECURSIVE')}">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                </node>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:if test="$debug = 1">
                                                  <xsl:message>Parent: <xsl:value-of select="$this"
                                                  /> - <xsl:value-of select="$thisParent"
                                                  /></xsl:message>
                                                </xsl:if>
                                                <xsl:apply-templates select="$newParent[1]"
                                                  mode="CAM">
                                                  <xsl:with-param name="shortname"
                                                  select="$shortname"/>
                                                  <xsl:with-param name="dictionary"
                                                  select="$dictionary"/>
                                                  <xsl:with-param name="itemPath"
                                                  select="concat($itemPath,$this,'|')"/>
                                                  <xsl:with-param name="parent" select="$thisParent"
                                                  />
                                                </xsl:apply-templates>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:when>
                                    <xsl:when
                                        test="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$context)]">
                                        <xsl:variable name="newParent"
                                            select="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$context)]"/>
                                        <xsl:choose>
                                            <xsl:when
                                                test="contains($itemPath,concat('|',$this,'|'))">
                                                <!-- Already in parent, so prevent looping -->
                                                <xsl:message> - recursive parent <xsl:value-of
                                                  select="$this"/> - <xsl:value-of
                                                  select="$itemPath"/></xsl:message>
                                                <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id($itself)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="bubble" TEXT="{concat($this,' RECURSIVE')}">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                </node>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:if test="$debug = 1">
                                                  <xsl:message>- Parents: <xsl:value-of
                                                  select="$this"/> - <xsl:value-of
                                                  select="$thisParent"/></xsl:message>
                                                </xsl:if>
                                                <xsl:apply-templates select="$newParent[1]"
                                                  mode="CAM">
                                                  <xsl:with-param name="shortname"
                                                  select="$shortname"/>
                                                  <xsl:with-param name="dictionary"
                                                  select="$dictionary"/>
                                                  <xsl:with-param name="itemPath"
                                                  select="concat($itemPath,$this,'|')"/>
                                                  <xsl:with-param name="parent" select="$thisParent"
                                                  />
                                                </xsl:apply-templates>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:when>
                                    <xsl:when
                                        test="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$parentContext)]">
                                        <xsl:variable name="newParent"
                                            select="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$parentContext)]"/>
                                        <xsl:choose>
                                            <xsl:when
                                                test="contains($itemPath,concat('|',$this,'|'))">
                                                <!-- Already in parent, so prevent looping -->
                                                <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id($itself)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="bubble" TEXT="{concat($this,' RECURSIVE')}">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                </node>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:if test="$debug = 1">
                                                  <xsl:message>- SingleParent: <xsl:value-of
                                                  select="$this"/> - <xsl:value-of
                                                  select="$singleContext"/></xsl:message>
                                                </xsl:if>
                                                <xsl:apply-templates select="$newParent[1]"
                                                  mode="CAM">
                                                  <xsl:with-param name="shortname"
                                                  select="$shortname"/>
                                                  <xsl:with-param name="dictionary"
                                                  select="$dictionary"/>
                                                  <xsl:with-param name="itemPath"
                                                  select="concat($itemPath,$this,'|')"/>
                                                  <xsl:with-param name="parent"
                                                  select="$singleContext"/>
                                                </xsl:apply-templates>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:when>
                                    <xsl:when
                                        test="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$fullContext)]">
                                        <xsl:variable name="newParent"
                                            select="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$fullContext)]"/>
                                        <xsl:choose>
                                            <xsl:when
                                                test="contains($itemPath,concat('|',$this,'|'))">
                                                <!-- Already in parent, so prevent looping -->
                                                <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id($itself)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="bubble" TEXT="{concat($this,' RECURSIVE')}">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                </node>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:if test="$debug = 1">
                                                  <xsl:message>- ParentParent: <xsl:value-of
                                                  select="$this"/> - <xsl:value-of
                                                  select="$parentContext"/></xsl:message>
                                                </xsl:if>
                                                <xsl:apply-templates select="$newParent[1]"
                                                  mode="CAM">
                                                  <xsl:with-param name="shortname"
                                                  select="$shortname"/>
                                                  <xsl:with-param name="dictionary"
                                                  select="$dictionary"/>
                                                  <xsl:with-param name="itemPath"
                                                  select="concat($itemPath,$this,'|')"/>
                                                  <xsl:with-param name="parent"
                                                  select="$parentContext"/>
                                                </xsl:apply-templates>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:when>
                                    <xsl:when
                                        test="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$itemContext)]">
                                        <xsl:variable name="newParent"
                                            select="$dictionary//Property[@CCTS='ABIE'][@item=$this][starts-with(@context,$itemContext)]"/>
                                        <xsl:choose>
                                            <xsl:when
                                                test="contains($itemPath,concat('|',$this,'|'))">
                                                <!-- Already in parent, so prevent looping -->
                                                <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id($itself)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="bubble" TEXT="{concat($this,' RECURSIVE')}">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                </node>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:if test="$debug = 1">
                                                  <xsl:message>- ItemParent: <xsl:value-of
                                                  select="$this"/> - <xsl:value-of
                                                  select="$parentContext"/></xsl:message>
                                                </xsl:if>
                                                <xsl:apply-templates select="$newParent[1]"
                                                  mode="CAM">
                                                  <xsl:with-param name="shortname"
                                                  select="$shortname"/>
                                                  <xsl:with-param name="dictionary"
                                                  select="$dictionary"/>
                                                  <xsl:with-param name="itemPath"
                                                  select="concat($itemPath,$this,'|')"/>
                                                  <xsl:with-param name="parent"
                                                  select="$parentContext"/>
                                                </xsl:apply-templates>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:when>

                                    <xsl:when
                                        test="$dictionary//Property[@CCTS='ABIE'][@item=$this][contains(@context,$moreContext)]">
                                        <xsl:variable name="newParent"
                                            select="$dictionary//Property[@CCTS='ABIE'][@item=$this][contains(@context,$moreContext)]"/>
                                        <xsl:choose>
                                            <xsl:when
                                                test="contains($itemPath,concat('|',$this,'|'))">
                                                <!-- Already in parent, so prevent looping -->
                                                <node CREATED="{$EpochSeconds}"
                                                  ID="ID_{generate-id($itself)}"
                                                  MODIFIED="{$EpochSeconds}" POSITION="right"
                                                  STYLE="bubble" TEXT="{concat($this,' RECURSIVE')}">
                                                  <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                                </node>
                                            </xsl:when>
                                            <xsl:otherwise>
                                                <xsl:if test="$debug = 1">
                                                  <xsl:message>- MoreParent: <xsl:value-of
                                                  select="$this"/> - <xsl:value-of
                                                  select="$moreContext"/></xsl:message>
                                                </xsl:if>
                                                <xsl:apply-templates select="$newParent[1]"
                                                  mode="CAM">
                                                  <xsl:with-param name="shortname"
                                                  select="$shortname"/>
                                                  <xsl:with-param name="dictionary"
                                                  select="$dictionary"/>
                                                  <xsl:with-param name="itemPath"
                                                  select="concat($itemPath,$this,'|')"/>
                                                  <xsl:with-param name="parent" select="$thisParent"
                                                  />
                                                </xsl:apply-templates>
                                            </xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:when>
                                    <xsl:when
                                        test="$dictionary//Property[@CCTS='ABIE'][@item=$this]">
                                        <xsl:message>Error: <xsl:value-of select="$this"/> -
                                                <xsl:value-of select="$itemContext"/> -
                                                <xsl:value-of select="$itemPath"/></xsl:message>
                                        <node CREATED="{$EpochSeconds}"
                                            ID="ID_{generate-id($itself)}"
                                            MODIFIED="{$EpochSeconds}" POSITION="right"
                                            STYLE="bubble" TEXT="{concat($this,' ERROR')}">
                                            <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                                        </node>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:variable name="this" select="."/>
                                        <xsl:variable name="BBIEdetails"
                                            select="$dictionary//Property[@CCTS='BBIE'][@item=$this]"/>
                                        <node CREATED="{$EpochSeconds}"
                                            ID="ID_{generate-id($itself)}"
                                            MODIFIED="{$EpochSeconds}" POSITION="right" STYLE="fork"
                                            TEXT="{$this}">
                                            <font BOLD="false" NAME="SansSerif" SIZE="12"/>
                                            <xsl:variable name="content">
                                                <xsl:if test="$BBIEdetails[1]/@mask &gt; ''">
                                                  <attribute NAME="{$BBIEdetails[1]/@type}"
                                                  VALUE="{$BBIEdetails[1]/@mask}"/>
                                                </xsl:if>
                                            </xsl:variable>
                                            <xsl:if
                                                test="$content/attribute and(contains($details,'content'))">
                                                <xsl:copy-of select="$content"/>
                                            </xsl:if>
                                            <xsl:if
                                                test="string-length($BBIEdetails[1]/@definition) &gt; 0 and(contains($details,'description'))">
                                                <richcontent TYPE="NOTE">
                                                  <html>
                                                  <head/>
                                                  <body>
                                                  <p>
                                                  <xsl:value-of select="$BBIEdetails[1]/@definition"
                                                  />
                                                  </p>
                                                  </body>
                                                  </html>
                                                </richcontent>
                                            </xsl:if>
                                        </node>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:otherwise/>
                            <!-- ignore this item -->
                        </xsl:choose>
                    </xsl:for-each>
                </node>
            </xsl:otherwise>
        </xsl:choose>

        <!-- xsl:apply-templates select="@*" mode="CAM"/>
            <xsl:apply-templates select="*" mode="CAM"/>
            <xsl:apply-templates select="text()[1]" mode="CAM"/ -->
    </xsl:template>

    <xsl:template match="@*" mode="CAM">
        <xsl:message>Attribute <xsl:value-of select="name()"/></xsl:message>
    </xsl:template>

    <xsl:template match="Parent" mode="compiled">
        <xsl:param name="dictionaryFile"/>
        <xsl:variable name="cardinality">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="@repeatable='true'">*</xsl:if>
                <xsl:if test="@optional='true'">?</xsl:if>
            </xsl:if>
        </xsl:variable>
        <xsl:variable name="displayFold">
            <xsl:choose>
                <xsl:when test="Parent">true</xsl:when>
                <xsl:when test="Child">true</xsl:when>
                <xsl:when test="Attribute">true</xsl:when>
                <xsl:otherwise>false</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}" MODIFIED="{$EpochSeconds}"
            POSITION="right" STYLE="bubble" FOLDED="{$displayFold}"
            TEXT="{concat,@name,' ',$cardinality}" xsl:exclude-result-prefixes="#all">
            <font BOLD="true" NAME="SansSerif" SIZE="12"/>
            <xsl:if test="contains($details,'content')">
                <xsl:variable name="findUID" select="@UID"/>
                <xsl:variable name="Rules" select="$dictionaryFile//Rules/Rule[@UID=$findUID]"/>
                <xsl:for-each select="$Rules">
                    <xsl:if test="@type">
                        <attribute NAME="type" VALUE="{@type}"/>
                    </xsl:if>
                    <xsl:if test="@mask">
                        <attribute NAME="mask" VALUE="{@mask}"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:if>
            <xsl:apply-templates select="*" mode="compiled">
                <xsl:with-param name="dictionaryFile" select="$dictionaryFile"/>
            </xsl:apply-templates>
        </node>
    </xsl:template>

    <xsl:template match="Child" mode="compiled">
        <xsl:param name="dictionaryFile"/>
        <xsl:variable name="cardinality">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="@repeatable='true'">*</xsl:if>
                <xsl:if test="@optional='true'">?</xsl:if>
            </xsl:if>
        </xsl:variable>
        <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}" MODIFIED="{$EpochSeconds}"
            POSITION="right" STYLE="bubble" FOLDED="false" TEXT="{concat,@name,' ',$cardinality}"
            xsl:exclude-result-prefixes="#all">
            <font BOLD="true" NAME="SansSerif" SIZE="12"/>
            <xsl:if test="contains($details,'content')">
                <xsl:variable name="findUID" select="@UID"/>
                <xsl:variable name="Rules" select="$dictionaryFile//Rules/Rule[@UID=$findUID]"/>
                <xsl:for-each select="$Rules">
                    <xsl:if test="@type">
                        <attribute NAME="type" VALUE="{@type}"/>
                    </xsl:if>
                    <xsl:if test="@mask">
                        <attribute NAME="mask" VALUE="{@mask}"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:if>
            <xsl:apply-templates select="*" mode="compiled">
                <xsl:with-param name="dictionaryFile" select="$dictionaryFile"/>
            </xsl:apply-templates>
        </node>
    </xsl:template>

    <xsl:template match="Attribute" mode="compiled">
        <xsl:param name="dictionaryFile"/>
        <xsl:variable name="cardinality">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="@repeatable='true'">*</xsl:if>
                <xsl:if test="@optional='true'">?</xsl:if>
            </xsl:if>
        </xsl:variable>
        <node CREATED="{$EpochSeconds}" ID="ID_{generate-id(.)}" MODIFIED="{$EpochSeconds}"
            POSITION="right" STYLE="fork" FOLDED="false" TEXT="@{concat,@name,' ',$cardinality}"
            xsl:exclude-result-prefixes="#all">
            <font BOLD="true" NAME="SansSerif" SIZE="12"/>
            <xsl:if test="contains($details,'content')">
                <xsl:variable name="findUID" select="@UID"/>
                <xsl:variable name="Rules" select="$dictionaryFile//Rules/Rule[@UID=$findUID]"/>
                <xsl:for-each select="$Rules">
                    <xsl:if test="@type">
                        <attribute NAME="type" VALUE="{@type}"/>
                    </xsl:if>
                    <xsl:if test="@mask">
                        <attribute NAME="mask" VALUE="{@mask}"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:if>
        </node>
    </xsl:template>

    <xsl:template match="Parent" mode="XMI">
        <!-- XMI branch level nodes -->
        <!-- xsl:message> - <xsl:value-of select="@name"/></xsl:message>
        <xsl:message> - <xsl:copy-of select="."/></xsl:message -->
        <xsl:variable name="xpath" select="concat('/',parent::node()/@name,'/',@name)"/>
        <xsl:variable name="cardinality">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="$rules/Rule[@type='repeatable'][ends-with(@xpath,$xpath)]">*</xsl:if>
                <xsl:if test="$rules/Rule[@type='optional'][ends-with(@xpath,$xpath)]">?</xsl:if>
            </xsl:if>
        </xsl:variable>
        <xsl:variable name="content">
            <xsl:if test="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]">
                <xsl:variable name="type"
                    select="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@type"/>
                <xsl:variable name="mask"
                    select="if($rules/Rule[@values][ends-with(@xpath,$xpath)]) then $rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@values else $rules/Rule[@mask][ends-with(@xpath,$xpath)]/@mask"/>
                <attribute NAME="{$type[1]}" VALUE="{$mask[1]}"/>
            </xsl:if>
        </xsl:variable>
        <xsl:variable name="description" select="@definition"/>
        <xsl:variable name="excluded"
            select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''"/>
        <xsl:variable name="uuid" select="uuid:randomUUID()"/>
        <packagedElement xmi:type="uml:Package" xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="{@name}">
            <xsl:if test="not($cardinality = '')">
                <uml:Multiplicity>
                    <uml:Multiplicity.range>
                        <uml:MultiplicityRange
                            lower="{if(contains($cardinality,'?')) then '0' else '1'}"
                            upper="{if(contains($cardinality,'*')) then '-1' else '1'}"/>
                    </uml:Multiplicity.range>
                </uml:Multiplicity>
            </xsl:if>

            <!-- output any details -->
            <xsl:call-template name="outputXMIdetails">
                <xsl:with-param name="xpath" select="$xpath"/>
                <xsl:with-param name="version" select="'2.1'"/>
                <xsl:with-param name="parent-uuid" select="$uuid"/>
                <xsl:with-param name="definition" select="@definition"/>
                <xsl:with-param name="mode">uml:Package</xsl:with-param>
            </xsl:call-template>

            <xsl:variable name="xmiMode">uml:Package</xsl:variable>
            <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                    <font BOLD="true" NAME="SansSerif" SIZE="12"/>
                    <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
                    <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if>
                    <xsl:if test="$content/attribute and(contains($details,'content'))"><xsl:copy-of select="$content"/></xsl:if>
                    <xsl:if test="$description[1] &gt; ' ' and(contains($details,'description'))"><richcontent TYPE="NOTE"><html><head/><body><p><xsl:value-of select="$description[1]"/></p></body></html></richcontent></xsl:if -->
            <xsl:if test="Attribute">
                <xsl:variable name="abstract-uuid" select="uuid:randomUUID()"/>
                <packagedElement xmi:type="uml:Class" xmi:id="{$abstract-uuid}"
                    name="{@name}_attributes" visibility="public" association="{$uuid}">
                    <xsl:for-each select="Attribute">
                        <xsl:variable name="xpath" select="concat($xpath,'/@',@name)"/>
                        <xsl:variable name="excluded"
                            select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''"/>
                        <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                        <ownedAttribute xmi:type="uml:Property" xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                            name="{@name}" visibility="public"
                            type="{as-xsl:uuid-type($rules/Rule[ends-with(@xpath,$xpath)],'child')}">
                            <!-- output any annotations -->
                            <xsl:call-template name="outputXMIdetails">
                                <xsl:with-param name="xpath" select="$xpath"/>
                                <xsl:with-param name="version" select="'2.1'"/>
                                <xsl:with-param name="parent-uuid" select="$uuid"/>
                                <xsl:with-param name="definition" select="@definition"/>
                                <xsl:with-param name="mode" select="$xmiMode"/>
                            </xsl:call-template>
                        </ownedAttribute>
                        <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                        <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                        <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
                        <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if -->
                    </xsl:for-each>
                </packagedElement>
            </xsl:if>
            <xsl:if test="$excluded = ''">
                <xsl:apply-templates select="Parent|Child" mode="XMI"/>
            </xsl:if>

        </packagedElement>
        <!-- node CREATED="1300763027494" ID="ID_1080794257" MODIFIED="1300763055369" TEXT="Drag and Drop Designer"/ -->

    </xsl:template>


    <xsl:template match="Child" mode="XMI">
        <!-- leaf nodes -->
        <xsl:variable name="xpath" select="concat('/',parent::node()/@name,'/',@name)"/>
        <xsl:variable name="description" select="@definition"/>
        <xsl:variable name="excluded"
            select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''"/>
        <xsl:variable name="content">
            <xsl:if test="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]">
                <xsl:variable name="type"
                    select="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@type"/>
                <xsl:variable name="mask"
                    select="if($rules/Rule[@values][ends-with(@xpath,$xpath)]) then $rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@values else $rules/Rule[@mask][ends-with(@xpath,$xpath)]/@mask"/>
                <attribute NAME="{$type[1]}" VALUE="{$mask[1]}"/>
            </xsl:if>
        </xsl:variable>
        <xsl:variable name="uuid" select="uuid:randomUUID()"/>
        <xsl:variable name="cardinality">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="$rules/Rule[@type='repeatable'][ends-with(@xpath,$xpath)]">*</xsl:if>
                <xsl:if test="$rules/Rule[@type='optional'][ends-with(@xpath,$xpath)]">?</xsl:if>
            </xsl:if>
        </xsl:variable>

        <packagedElement xmi:type="uml:Class" xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="{@name}">
            <xsl:if test="not($cardinality = '')">
                <uml:Multiplicity>
                    <uml:Multiplicity.range>
                        <uml:MultiplicityRange
                            lower="{if(contains($cardinality,'?')) then '0' else '1'}"
                            upper="{if(contains($cardinality,'*')) then '-1' else '1'}"/>
                    </uml:Multiplicity.range>
                </uml:Multiplicity>
            </xsl:if>
            <xsl:copy-of select="as-xsl:content($content)"/>
            <!-- output data type model -->

            <xsl:variable name="xpath" select="concat('/',parent::node()/@name,'/',@name)"/>
            <xsl:variable name="description" select="@definition"/>
            <xsl:variable name="excluded"
                select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''"/>
            <xsl:variable name="xmiMode">uml:Class</xsl:variable>
            <xsl:for-each select="Attribute">
                <xsl:variable name="xpath" select="concat('/',parent::node()/@name,'/@',@name)"/>
                <xsl:variable name="excluded"
                    select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''"/>
                <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                <ownedAttribute xmi:type="uml:Property" xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                    name="{@name}" visibility="public"
                    type="{as-xsl:uuid-type($rules/Rule[ends-with(@xpath,$xpath)],'child')}">
                    <xsl:call-template name="outputXMIdetails">
                        <xsl:with-param name="xpath" select="$xpath"/>
                        <xsl:with-param name="version" select="'2.1'"/>
                        <xsl:with-param name="parent-uuid" select="$uuid"/>
                        <xsl:with-param name="definition" select="@definition"/>
                        <xsl:with-param name="mode" select="$xmiMode"/>
                    </xsl:call-template>
                </ownedAttribute>
                <!-- xsl:if test="not($excluded='')"><xsl:attribute name="COLOR">#ff6666</xsl:attribute></xsl:if>
                    <font BOLD="false" NAME="SansSerif" SIZE="10"/>
                    <xsl:if test="not($excluded='')"><icon BUILTIN="button_cancel"/></xsl:if>
                    <xsl:if test="count($descriptions/Key[ends-with(@xpath,$xpath)]) &gt; 0"><icon BUILTIN="password"/></xsl:if -->
            </xsl:for-each>

        </packagedElement>

    </xsl:template>

    <xsl:template name="outputXMIdetails">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <xsl:param name="definition"/>
        <xsl:param name="mode"/>

        <xsl:call-template name="outputXMIcomment">
            <xsl:with-param name="xpath" select="$xpath"/>
            <xsl:with-param name="version" select="'2.1'"/>
            <xsl:with-param name="parent-uuid" select="$parent-uuid"/>
            <xsl:with-param name="definition" select="$definition"/>
        </xsl:call-template>

        <xsl:call-template name="outputXMIrule">
            <xsl:with-param name="xpath" select="$xpath"/>
            <xsl:with-param name="version" select="'2.1'"/>
            <xsl:with-param name="parent-uuid" select="$parent-uuid"/>
        </xsl:call-template>

        <xsl:if test="$mode='uml:Class'">
            <xsl:call-template name="outputXMItype">
                <xsl:with-param name="xpath" select="$xpath"/>
                <xsl:with-param name="version" select="'2.1'"/>
                <xsl:with-param name="parent-uuid" select="$parent-uuid"/>
            </xsl:call-template>
        </xsl:if>

        <xsl:if test="$mode='uml:Class'">
            <xsl:call-template name="outputXMIenumeration">
                <xsl:with-param name="xpath" select="$xpath"/>
                <xsl:with-param name="version" select="'2.1'"/>
                <xsl:with-param name="parent-uuid" select="$parent-uuid"/>
            </xsl:call-template>
        </xsl:if>

    </xsl:template>

    <xsl:template name="outputXMIcomment">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <xsl:param name="definition"/>
        <!-- output any annotations -->
        <xsl:if test="contains($details,'description') and(string-length($definition) &gt; 0)">
            <xsl:choose>
                <xsl:when test="$version = '2.1'">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <xsl:variable name="description" select="$definition"/>
                    <ownedComment xmi:type="uml:Comment" xmi:id="{$uuid}" xmi:uuid="{$uuid}">
                        <body>
                            <xsl:value-of select="$description"/>
                        </body>
                        <annotatedElement xmi:idref="{$parent-uuid}"/>
                    </ownedComment>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message> - warning: annotation for <xsl:value-of select="$xpath"/> -
                        unknown xmi version: [<xsl:value-of select="$version"/>]</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template name="outputXMIrule">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <!-- output any annotations -->
        <xsl:if
            test="contains($details,'cardinality') and($rules/Rule[@type='optional'][ends-with(@xpath,$xpath)])">
            <xsl:choose>
                <xsl:when test="$version = '2.1'">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <ownedRule xmi:type="uml:Constraint" xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                        name="optional">
                        <constrainedElement xmi:idref="{$parent-uuid}"/>
                    </ownedRule>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message> - warning: annotation for <xsl:value-of select="$xpath"/> -
                        unknown xmi version: [<xsl:value-of select="$version"/>]</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template name="outputXMItype">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <!-- output any annotations -->
        <xsl:if
            test="contains($details,'content') and($rules/Rule[not(@type='optional')][ends-with(@xpath,$xpath)])">
            <xsl:choose>
                <xsl:when test="$version = '2.1'">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <xsl:variable name="typeOf"
                        select="$rules/Rule[not(@type='optional')][ends-with(@xpath,$xpath)]/@type"/>
                    <nestedClassifier xmi:type="uml:DataType" xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                        name="{$typeOf[1]}"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message> - warning: annotation for <xsl:value-of select="$xpath"/> -
                        unknown xmi version: [<xsl:value-of select="$version"/>]</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template name="outputXMIenumeration">
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <xsl:param name="parent-uuid"/>
        <!-- output any annotations -->
        <xsl:if
            test="contains($details,'content') and($rules/Rule[@values][ends-with(@xpath,$xpath)])">
            <xsl:choose>
                <xsl:when test="$version = '2.1'">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <xsl:variable name="ruleValues"
                        select="$rules/Rule[@values][ends-with(@xpath,$xpath)]/@values"/>
                    <nestedClassifier xmi:type="uml:Enumeration" xmi:id="{$uuid}" xmi:uuid="{$uuid}"
                        name="list">
                        <xsl:for-each select="tokenize($ruleValues[1],''' \|''')">
                            <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                            <ownedLiteral xmi:type="uml:EnumerationLiteral" xmi:id="{$uuid}"
                                xmi:uuid="{$uuid}" name="{replace(.,'''','')}"/>
                        </xsl:for-each>
                    </nestedClassifier>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:message> - warning: annotation for <xsl:value-of select="$xpath"/> -
                        unknown xmi version: [<xsl:value-of select="$version"/>]</xsl:message>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:template>

    <xsl:template match="Parent" mode="ArgoUML">
        <!-- Argo XMI branch level nodes -->

        <xsl:variable name="xpath" select="concat('/',parent::node()/@name,'/',@name)"/>
        <xsl:variable name="cardinality">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="$rules/Rule[@type='repeatable'][ends-with(@xpath,$xpath)]">*</xsl:if>
                <xsl:if test="$rules/Rule[@type='optional'][ends-with(@xpath,$xpath)]">?</xsl:if>
            </xsl:if>
        </xsl:variable>
        <!-- xsl:variable name="content">
                <xsl:if test="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]">
                <xsl:variable name="type" select="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@type"/>
                <xsl:variable name="mask" select="if($rules/Rule[@values][ends-with(@xpath,$xpath)]) then $rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@values else $rules/Rule[@mask][ends-with(@xpath,$xpath)]/@mask"></xsl:variable>
                <attribute NAME="{$type[1]}" VALUE="{$mask[1]}"/>
                </xsl:if>
                </xsl:variable -->
        <xsl:variable name="description" select="@definition"/>
        <xsl:variable name="excluded"
            select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''"/>
        <xsl:variable name="uuid" select="uuid:randomUUID()"/>
        <UML:Namespace.ownedElement>
            <UML:Class xmi.id="{$uuid}" name="{@name}" visibility="public" isSpecification="false"
                isRoot="true" isLeaf="false" isAbstract="false" isActive="false">
                <xsl:if test="contains($details,'description') and(@definition)">
                    <xsl:variable name="uuid-desc" select="uuid:randomUUID()"/>
                    <UML:ModelElement.taggedValue>
                        <UML:TaggedValue xmi.id="{$uuid-desc}" isSpecification="false">
                            <UML:TaggedValue.dataValue>
                                <xsl:value-of select="$description"/>
                            </UML:TaggedValue.dataValue>
                            <xsl:copy-of select="$ArgoUML-documentation"/>
                        </UML:TaggedValue>
                    </UML:ModelElement.taggedValue>
                </xsl:if>

                <xsl:if test="not($cardinality = '')">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <UML:StructuralFeature.multiplicity>
                        <UML:Multiplicity xmi.id="{$uuid}">
                            <UML:Multiplicity.range>
                                <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                                <UML:MultiplicityRange xmi.id="{$uuid}"
                                    lower="{if(contains($cardinality,'?')) then '0' else '1'}"
                                    upper="{if(contains($cardinality,'*')) then '-1' else '1'}"/>
                            </UML:Multiplicity.range>
                        </UML:Multiplicity>
                    </UML:StructuralFeature.multiplicity>
                </xsl:if>

                <!-- output any details -->
                <xsl:call-template name="outputArgoUMLdetails">
                    <xsl:with-param name="xpath" select="$xpath"/>
                    <xsl:with-param name="version" select="'2.1'"/>
                    <xsl:with-param name="parent-uuid" select="$uuid"/>
                </xsl:call-template>

                <!-- xsl:for-each select="@*">
                        <xsl:variable name="xpath" select="concat('/',name(parent::node()),'/@',name())"/>
                        <xsl:variable name="excluded" select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''" />
                        <xsl:variable name="uuid" select="uuid:randomUUID()"/ -->

                <!-- output any annotations -->
                <!-- ownedAttribute xmi:type="uml:Property" xmi:id="{$uuid}" xmi:uuid="{$uuid}" name="{name(.)}" visibility="public" type="{as-xsl:uuid-type($rules/Rule[ends-with(@xpath,$xpath)],'child')}">
                        <xsl:call-template name="outputArgoUMLdetails">
                        <xsl:with-param name="xpath" select="$xpath"/>
                        <xsl:with-param name="version" select="'2.1'"/>
                        <xsl:with-param name="parent-uuid" select="$uuid"/>
                        </xsl:call-template>
                        </ownedAttribute -->
                <!-- /xsl:for-each -->
                <xsl:if test="$excluded = ''">
                    <xsl:apply-templates select="Parent|Child" mode="ArgoUML"/>
                </xsl:if>

            </UML:Class>
        </UML:Namespace.ownedElement>
        <!-- node CREATED="1300763027494" ID="ID_1080794257" MODIFIED="1300763055369" TEXT="Drag and Drop Designer"/ -->


    </xsl:template>

    <xsl:template match="Child" mode="ArgoUML">
        <!-- leaf nodes -->
        <xsl:variable name="xpath" select="concat('/',parent::node()/@name,'/',@name)"/>
        <xsl:variable name="description" select="@definition"/>
        <xsl:variable name="excluded"
            select="if(count($excludes/Rule[ends-with(@xpath,$xpath)]) &gt; 0) then ' EXCLUDED' else ''"/>
        <!-- xsl:variable name="content">
            <xsl:if test="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]">
            <xsl:variable name="type" select="$rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@type"/>
            <xsl:variable name="mask" select="if($rules/Rule[@values][ends-with(@xpath,$xpath)]) then $rules/Rule[@mask|@values][ends-with(@xpath,$xpath)]/@values else $rules/Rule[@mask][ends-with(@xpath,$xpath)]/@mask"></xsl:variable>
            <attribute NAME="{$type[1]}" VALUE="{$mask[1]}"/>
            </xsl:if>
            </xsl:variable -->
        <xsl:variable name="class_uuid" select="uuid:randomUUID()"/>
        <xsl:variable name="cardinality">
            <xsl:if test="contains($details,'cardinality')">
                <xsl:if test="$rules/Rule[@type='repeatable'][ends-with(@xpath,$xpath)]">*</xsl:if>
                <xsl:if test="$rules/Rule[@type='optional'][ends-with(@xpath,$xpath)]">?</xsl:if>
            </xsl:if>
        </xsl:variable>

        <UML:Namespace.ownedElement>
            <UML:Class xmi.id="{$class_uuid}" name="{@name}" visibility="public"
                isSpecification="false" isRoot="false" isLeaf="false" isAbstract="false"
                isActive="false">
                <xsl:if test="contains($details,'description') and(@definition)">
                    <xsl:variable name="uuid-desc" select="uuid:randomUUID()"/>
                    <UML:ModelElement.taggedValue>
                        <UML:TaggedValue xmi.id="{$uuid-desc}" isSpecification="false">
                            <UML:TaggedValue.dataValue>
                                <xsl:value-of select="$description"/>
                            </UML:TaggedValue.dataValue>
                            <xsl:copy-of select="$ArgoUML-documentation"/>
                        </UML:TaggedValue>
                    </UML:ModelElement.taggedValue>
                </xsl:if>
                <!-- UML:Attribute xmi.id = "{$uuid}"
                    name = "{name(.)}" visibility = "public" isSpecification = "false"
                    ownerScope = "instance" changeability = "changeable" targetScope = "instance" -->
                <xsl:if test="not($cardinality = '')">
                    <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                    <!-- UML:StructuralFeature.multiplicity>
                        <UML:Multiplicity xmi.id = "{$uuid}">
                        <UML:Multiplicity.range>
                        <xsl:variable name="uuid" select="uuid:randomUUID()"/>
                        <UML:MultiplicityRange xmi.id = "{$uuid}"
                        lower = "{if(contains($cardinality,'?')) then '0' else '1'}" upper = "{if(contains($cardinality,'*')) then '-1' else '1'}"/>
                        </UML:Multiplicity.range>
                        </UML:Multiplicity>
                        </UML:StructuralFeature.multiplicity -->
                    <UML:ModelElement.taggedValue>
                        <UML:TaggedValue xmi.id="{$uuid}" isSpecification="false">
                            <UML:TaggedValue.dataValue>Cardinality: lower=<xsl:value-of
                                    select="if(contains($cardinality,'?')) then '0' else '1'"/>
                                    upper=<xsl:value-of
                                    select="if(contains($cardinality,'*')) then '-1' else '1'"
                                /></UML:TaggedValue.dataValue>
                            <xsl:copy-of select="$ArgoUML-usage"/>
                        </UML:TaggedValue>
                    </UML:ModelElement.taggedValue>
                </xsl:if>
                <UML:StructuralFeature.type>
                    <UML:Class xmi.idref="{$text_uuid}"/>
                </UML:StructuralFeature.type>

            </UML:Class>
        </UML:Namespace.ownedElement>
    </xsl:template>

    <xsl:template name="outputArgoUMLdetails">
        <xsl:param name="parent-uuid"/>
        <xsl:param name="xpath"/>
        <xsl:param name="version"/>
        <!-- empty for now -->
    </xsl:template>


    <xsl:function name="as-xsl:uuid-type">
        <xsl:param name="rules"/>
        <xsl:param name="nodeType"/>

        <xsl:choose>
            <xsl:when test="$rules[@type = 'list']">
                <xsl:value-of select="$list_uuid"/>
            </xsl:when>
            <xsl:when test="$rules[@type = 'number']">
                <xsl:value-of select="$decimal_uuid"/>
            </xsl:when>
            <xsl:when test="$rules[@type = 'datetype']">
                <xsl:value-of select="$datetime_uuid"/>
            </xsl:when>
            <xsl:when test="$nodeType = 'child'">
                <xsl:value-of select="$text_uuid"/>
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>

    </xsl:function>

    <xsl:function name="as-xsl:uuid-dicttype">
        <xsl:param name="data-type"/>
        <xsl:choose>
            <xsl:when test="$data-type='string'">
                <xsl:value-of select="$text_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='normalizedString'">
                <xsl:value-of select="$text_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='token'">
                <xsl:value-of select="$token_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='NMToken'">
                <xsl:value-of select="$NMToken_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='ID'">
                <xsl:value-of select="$ID_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='IDREF'">
                <xsl:value-of select="$IDREF_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='date'">
                <xsl:value-of select="$date_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='datetime'">
                <xsl:value-of select="$datetime_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='time'">
                <xsl:value-of select="$time_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='duration'">
                <xsl:value-of select="$duration_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='decimal'">
                <xsl:value-of select="$decimal_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='double'">
                <xsl:value-of select="$double_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='float'">
                <xsl:value-of select="$float_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='integer' or($data-type='int' )">
                <xsl:value-of select="$integer_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='byte'">
                <xsl:value-of select="$byte_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='boolean'">
                <xsl:value-of select="$boolean_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='long'">
                <xsl:value-of select="$long_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='short'">
                <xsl:value-of select="$short_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='list'">
                <xsl:value-of select="$list_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='NCName'">
                <xsl:value-of select="$NCName_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='QName'">
                <xsl:value-of select="$QName_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='any'">
                <xsl:value-of select="$any_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='anyAttribute'">
                <xsl:value-of select="$any_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='anyURI'">
                <xsl:value-of select="$uri_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='language'">
                <xsl:value-of select="$language_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='base64Binary'">
                <xsl:value-of select="$binary_uuid"/>
            </xsl:when>
            <xsl:when test="$data-type='hexBinary'">
                <xsl:value-of select="$binary_uuid"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$text_uuid"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="as-xsl:content">
        <xsl:param name="content"/>
        <xsl:if test="count($content/attribute) &gt; 0">
            <xsl:variable name="contentType">
                <xsl:choose>
                    <xsl:when test="$content/attribute[@NAME = 'list']">
                        <xsl:value-of select="$list_uuid"/>
                    </xsl:when>
                    <xsl:when test="$content/attribute[@NAME = 'number']">
                        <xsl:value-of select="$decimal_uuid"/>
                    </xsl:when>
                    <xsl:when test="$content/attribute[@NAME = 'datetype']">
                        <xsl:value-of select="$datetime_uuid"/>
                    </xsl:when>
                    <xsl:otherwise/>
                </xsl:choose>
            </xsl:variable>
            <uml:StructuralFeature.type>
                <uml:Class xmi.idref="{$contentType}"/>
            </uml:StructuralFeature.type>
        </xsl:if>
    </xsl:function>

</xsl:stylesheet>
