<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
      xmlns:xs="http://www.w3.org/2001/XMLSchema"
      exclude-result-prefixes="xs"
      version="2.0">
    
    <xsl:param name="lookup">EML-v5-dictionary-stuff.xml</xsl:param>
    
    <xsl:variable name="eml_lookup" select="document($lookup)"/>
    <xsl:output method="xml" indent="yes"></xsl:output>
<xsl:template match="/">
    <xsl:message>Lookup items:[<xsl:value-of select="count($eml_lookup/EML-doc/Entry)"/>]</xsl:message>
    <CCTScatalogue>
        <xsl:comment><xsl:value-of select="descendant::comment()"/></xsl:comment>
        <xsl:apply-templates select="//CCTSitem"/>
    </CCTScatalogue>
</xsl:template>
    
    <xsl:template match="CCTSitem">
        <xsl:variable name="names" select="tokenize(@dictionary-entry-name,'\.')"/>
        <xsl:variable name="this" select="$names[count($names)]"/>
        <xsl:variable name="defn" select="@definition"/>
        <xsl:variable name="definition">
            <xsl:choose>
                <xsl:when test="string-length(@definition) &gt; 0">
                    <xsl:value-of select="@definition"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:variable name="DEFN" select="$eml_lookup/EML-doc/Entry[@Name=$this]"/>
                    <xsl:value-of select="$DEFN[1]/@Definition"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:choose>
            <xsl:when test="string-length(@definition) &gt; 0">
                <xsl:message>Verify - <xsl:value-of select="@dictionary-entry-name"/>
                <xsl:if test="$eml_lookup/EML-doc/Entry[@Name=$this and(@Definition=$defn)]"> duplicate</xsl:if>
                <xsl:if test="$eml_lookup/EML-doc/Entry[@Name=$this and(not(@Definition=$defn))]"> DIFFERENT</xsl:if>
                    <xsl:if test="count($eml_lookup/EML-doc/Entry[@Name=$this]) = 0"> OK</xsl:if>
                 </xsl:message>
            </xsl:when>
            <xsl:otherwise>
                <xsl:message>Find - <xsl:value-of select="@dictionary-entry-name"/>
                <xsl:if test="$eml_lookup/EML-doc/Entry[@Name=$this and (string-length(@Definition) &gt; 0)]"> found</xsl:if>
                    <xsl:if test="$eml_lookup/EML-doc/Entry[@Name=$this and (string-length(@Definition) &lt; 1)]"> EMPTY</xsl:if>
                <xsl:if test="count($eml_lookup/EML-doc/Entry[@Name=$this]) = 0"> MISSING</xsl:if>
                </xsl:message>
            </xsl:otherwise>
        </xsl:choose>
                
        <CCTSitem uniqueID="{@uniqueID}" dictionary-entry-name="{@dictionary-entry-name}" type="{@type}"  definition="{$definition}" status="{@status}" 
            objectClass="{@objectClass}"
            property="{@property}"
            representation="{@representation}"/>    
        <xsl:if test="descendant::comment()"><xsl:comment><xsl:value-of select="descendant::comment()"/></xsl:comment></xsl:if>
    </xsl:template>
    
</xsl:stylesheet>
