<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:camed="http://jcam.org.uk/editor" xmlns:xsd="http://www.w3.org/2001/XMLSchema">

<!-- CAM utility functions -->
<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
     Authors: David RR Webber, June, 2009 -->
<!--
    Purpose  : Provide lists of keywords for renamer.xsl program
    Original : 06/09/2009
    Initials : drrw
    CAM ver    : 1.1 
    Description : Allows independent word lists to be managed.
    Updated  - Inits - Change
    06/09/09   drrw    Initial release
    06/10/09   drrw    More words
    08/23/09   drrw   Add common NIEM acronyms
    12/23/09   drrw   Add Guard abbreviations / acronyms
    01/11/10   drrw   Add RAMP abbreviations/ acronyms
    01/21/10   drrw   Add spelling check
    01/21/10   drrw   Add more EDM and RAMP abbreviations
    01/22/10   drrw   Add list of 200 most common mis-spellings from internet websites text
                                http://en.wikipedia.org/wiki/Commonly_misspelled_words#Documented_list_of_common_misspellings
    02/01/10  drrw    Add spelling words from camed:annotations
    03/04/10  drrw    Add abbreviations from EDXL
    03/12/10  drrw    Add EDM abbrevs and acronymns
    04/07/10  drrw    CEN abbreviations
    06/15/10  drrw    More abbrevations
-->
    
    <xsl:variable name="wordsList" select="camed:setSpellings()"/>

    <xsl:function name="camed:WordDefaults">
        <xsl:param name="name"/>
        <xsl:param name="nextname"/>
        <xsl:param name="mode"/>
        <xsl:variable name="nameUPPER" select="upper-case($name)"/>
        <xsl:choose>
            <xsl:when test="$mode='NIEM'">
                <xsl:choose>
                    <xsl:when test="$nameUPPER='ACCT'">Account</xsl:when>
                    <xsl:when test="$nameUPPER='URL'">URL</xsl:when>
                    <xsl:when test="$nameUPPER='URI'">URI</xsl:when>
                    <xsl:when test="$nameUPPER='URN'">URN</xsl:when>
                    <xsl:when test="$nameUPPER='AMT'">Amount</xsl:when>
                    <xsl:when test="$nameUPPER='CSE'">Case</xsl:when>
                    <xsl:when test="$nameUPPER='CSTDY'">Custody</xsl:when>
                    <xsl:when test="$nameUPPER='TXT'">Text</xsl:when>
                    <xsl:when test="$nameUPPER='TX'">Text</xsl:when>
                    <xsl:when test="$nameUPPER='TOT'">Total</xsl:when>
                    <xsl:when test="$nameUPPER='ENTRD'">Entered</xsl:when>
                    <xsl:when test="$nameUPPER='PSTL'">Postal</xsl:when>
                    <xsl:when test="$nameUPPER='STR'">Street</xsl:when>
                    <xsl:when test="$nameUPPER='BRTHPLC'">BirthPlace</xsl:when>
                    <xsl:when test="$nameUPPER='EXPRTN'">Expiration</xsl:when>
                    <xsl:when test="$nameUPPER='OCPTN'">Occupation</xsl:when>
                    <xsl:when test="$nameUPPER='DESC'">Description</xsl:when>
                    <xsl:when test="$nameUPPER='DESCN'">Description</xsl:when>
                    <xsl:when test="$nameUPPER='IND'">Indicator</xsl:when>
                    <xsl:when test="$nameUPPER='DT'">Date</xsl:when>
                    <xsl:when test="$nameUPPER='DTE'">Date</xsl:when>
                    <xsl:when test="$nameUPPER='DRVR'">Driver</xsl:when>
                    <xsl:when test="$nameUPPER='DPT'">Department</xsl:when>
                    <xsl:when test="$nameUPPER='FED'">Federal</xsl:when>
                    <xsl:when test="$nameUPPER='LOC'">Location</xsl:when>
                    <xsl:when test="$nameUPPER='EIN'">EIN</xsl:when>
                    <xsl:when test="$nameUPPER='OCPNT'">Occupation</xsl:when>
                    <xsl:when test="$nameUPPER='LCNS'">License</xsl:when>
                    <xsl:when test="$nameUPPER='RSDNT'">Resident</xsl:when>
                    <xsl:when test="$nameUPPER='NTRNT'">Entrant</xsl:when>
                    <xsl:when test="$nameUPPER='EMPLR'">Employer</xsl:when>
                    <xsl:when test="$nameUPPER='EMPLMT'">Employment</xsl:when>
                    <xsl:when test="$nameUPPER='EMPLMNT'">Employment</xsl:when>
                    <xsl:when test="$nameUPPER='NBR'">Number</xsl:when>
                    <xsl:when test="$nameUPPER='NUM'">Number</xsl:when>
                    <xsl:when test="$nameUPPER='NOTFD'">Notified</xsl:when>
                    <xsl:when test="$nameUPPER='QUATITY'">Quantity</xsl:when>
                    <xsl:when test="$nameUPPER='QUANT'">Quantity</xsl:when>
                    <xsl:when test="$nameUPPER='CD'">Code</xsl:when>
                    <xsl:when test="$nameUPPER='RSN'">Reason</xsl:when>
                    <xsl:when test="$nameUPPER='REASN'">Reason</xsl:when>
                    <xsl:when test="$nameUPPER='CMNT'">Comment</xsl:when>
                    <xsl:when test="$nameUPPER='CMT'">Comment</xsl:when>
                    <xsl:when test="$nameUPPER='GOVT'">Government</xsl:when>
                    <xsl:when test="$nameUPPER='PCKG'">Package</xsl:when>
                    <xsl:when test="$nameUPPER='DET'">Detention</xsl:when>
                    <xsl:when test="$nameUPPER='ITM'">Item</xsl:when>
                    <xsl:when test="$nameUPPER='WTCH'">Watch</xsl:when>
                    <xsl:when test="$nameUPPER='VAL'">Value</xsl:when>
                    <xsl:when test="$nameUPPER='FBI'">FBI</xsl:when>
                    <xsl:when test="$nameUPPER='UIC'">UIC</xsl:when>
                    <xsl:when test="$nameUPPER='FIPS'">FIPS</xsl:when>
                    <xsl:when test="$nameUPPER='ICE'">ICE</xsl:when>
                    <xsl:when test="$nameUPPER='ICEPIC'">ICEPIC</xsl:when>
                    <xsl:when test="$nameUPPER='GRP'">Group</xsl:when>
                    <xsl:when test="$nameUPPER='RCRD'">Record</xsl:when>
                    <xsl:when test="$nameUPPER='TYP'">Type</xsl:when>
                    <xsl:when test="$nameUPPER='FRST'">First</xsl:when>
                    <xsl:when test="$nameUPPER='NM'">Name</xsl:when>
                    <xsl:when test="$nameUPPER='MDL'">Middle</xsl:when>
                    <xsl:when test="$nameUPPER='LST'">Last</xsl:when>
                    <xsl:when test="$nameUPPER='INITL'">Initial</xsl:when>
                    <xsl:when test="$nameUPPER='OFC'">Office</xsl:when>
                    <xsl:when test="$nameUPPER='DSC'">Description</xsl:when>
                    <xsl:when test="$nameUPPER='ORG'">Organization</xsl:when>
                    <xsl:when test="$nameUPPER='PRSCTN'">Prosecution</xsl:when>
                    <xsl:when test="$nameUPPER='ACTG'">Acting</xsl:when>
                    <xsl:when test="$nameUPPER='SUPRVR'">Supervisor</xsl:when>
                    <xsl:when test="$nameUPPER='CAT'">Category</xsl:when>
                    <xsl:when test="$nameUPPER='DTTM'">DateTime</xsl:when>
                    <xsl:when test="$nameUPPER='ADDR'">Address</xsl:when>
                    <xsl:when test="$nameUPPER='TECS'">TECS</xsl:when>
                    <xsl:when test="$nameUPPER='USR' and(upper-case($nextname)='ID')">User</xsl:when>
                    <xsl:when test="$nameUPPER='INTL' and(upper-case($nextname)='CODE')">International</xsl:when>
                    <xsl:when test="$nameUPPER='INT' and(upper-case($nextname)='COUNTRY')">International</xsl:when>
                    <xsl:when test="$nameUPPER='CNTL' and(upper-case($nextname)='DATE' or(upper-case($nextname)='DT'))">Control</xsl:when>
                    <xsl:when test="$nameUPPER='CTY' and(upper-case($nextname)='NAME' or(upper-case($nextname)='NM') or(upper-case($nextname)='CD'))">City</xsl:when>
                    <xsl:when test="$nameUPPER='CN' and(upper-case($nextname)='NAME' or(upper-case($nextname)='NM') or(upper-case($nextname)='CD'))">Country</xsl:when>
                    <xsl:when test="$nameUPPER='BSN' and(upper-case($nextname)='NAME' or(upper-case($nextname)='QRY') or(upper-case($nextname)='NM') or(upper-case($nextname)='STUS'))">Business</xsl:when>
                    <xsl:when test="$nameUPPER='CERT'">Certificate</xsl:when>
                    <xsl:when test="$nameUPPER='TL' and(upper-case($nextname)='NUMBER' or(upper-case($nextname)='NBR'))">Tail</xsl:when>
                    <xsl:when test="$nameUPPER='STP' and(upper-case($nextname)='DATE' or(upper-case($nextname)='CC') or(upper-case($nextname)='DT'))">Stop</xsl:when>
                    <xsl:when test="$nameUPPER='NTFY'">Notify</xsl:when>
                    <xsl:when test="$nameUPPER='QRY'">Query</xsl:when>
                    <xsl:when test="$nameUPPER='ARN'"><xsl:value-of select="if(upper-case($nextname)='NUMBER' or(upper-case($nextname)='NBR' )) then 'AlienRegistration' else 'AlienRegistrationNumber'"/></xsl:when>
                    <xsl:when test="$nameUPPER='STRT'">Start</xsl:when>
                    <xsl:when test="$nameUPPER='NCRPRTN'">Incorporation</xsl:when>
                    <xsl:when test="$nameUPPER='SBLVL'">Sublevel</xsl:when>
                    <xsl:when test="$nameUPPER='TRMNL'">Terminal</xsl:when>
                    <xsl:when test="$nameUPPER='SRCH'">Search</xsl:when>
                    <xsl:when test="$nameUPPER='SEQ' and(upper-case($nextname)='NUMBER' or(upper-case($nextname)='NBR') or(upper-case($nextname)='CSE'))">Sequence</xsl:when>
                    <xsl:when test="$nameUPPER='NDRCOVR'">Undercover</xsl:when>
                    <xsl:when test="$nameUPPER='FAA'">FAA</xsl:when>
                    <xsl:when test="$nameUPPER='MFR'">Manufacturer</xsl:when>     
                    <xsl:when test="$nameUPPER='ENGN'">Engine</xsl:when>               
                    <xsl:when test="$nameUPPER='YR'">Year</xsl:when>               
                    <xsl:when test="$nameUPPER='ISUE'">Issue</xsl:when>                                   
                    <xsl:when test="$nameUPPER='ACTN'">Action</xsl:when>    
                    <xsl:when test="$nameUPPER='STUS'">Status</xsl:when>        
                    <xsl:when test="$nameUPPER='MK'">Make</xsl:when>     
                    <xsl:when test="$nameUPPER='GLBL'">Global</xsl:when>     
                    <xsl:when test="$nameUPPER='DETL'">Detail</xsl:when>   
                    <xsl:when test="$nameUPPER='ORGNL'">Original</xsl:when>   
                    <xsl:when test="$nameUPPER='MNFCTR'">Manufacturer</xsl:when>
                    <xsl:when test="$nameUPPER='OBJ'">Object</xsl:when>
                    <xsl:when test="$nameUPPER='ARTST'">Artist</xsl:when>
                    <xsl:when test="$nameUPPER='VHCL'">Vehicle</xsl:when>
                    <xsl:when test="$nameUPPER='VSL'">Vessel</xsl:when>
                    <xsl:when test="$nameUPPER='SHPYD'">Shipyard</xsl:when>
                    <xsl:when test="$nameUPPER='OWNR'">Owner</xsl:when>                    
                    <xsl:when test="$nameUPPER='PRMRY'">Primary</xsl:when>               
                    <xsl:when test="$nameUPPER='LKOUT'">Lookout</xsl:when>               
                    <xsl:when test="$nameUPPER='ACCES'">Access</xsl:when>      
                    <xsl:when test="$nameUPPER='MODL'">Model</xsl:when>    
                    <xsl:when test="$nameUPPER='SRL'">Serial</xsl:when>        
                    <xsl:when test="$nameUPPER='ASA'">ASA</xsl:when>                                   
                    <xsl:when test="$nameUPPER='RGSTRTN'">Registration</xsl:when>  
                    <xsl:when test="$nameUPPER='UPD'">Update</xsl:when>   
                    <xsl:when test="$nameUPPER='CTRY'">Country</xsl:when>   
                    <xsl:when test="$nameUPPER='CNTCT'">Contact</xsl:when>     
                    <xsl:when test="$nameUPPER='TM'">Time</xsl:when>      
                    <xsl:when test="$nameUPPER='MK'">Make</xsl:when>         
                    <xsl:when test="$nameUPPER='ALT'">Alternate</xsl:when>    
                    <xsl:when test="$nameUPPER='SRCE'">Source</xsl:when>                                   
                    <xsl:when test="$nameUPPER='WRTHY'">Worthy</xsl:when>                                   
                    <xsl:when test="$nameUPPER='LVL'">Level</xsl:when>     
                    <xsl:when test="$nameUPPER='FY'">FY</xsl:when>       
                    <xsl:when test="$nameUPPER='TITL'">Title</xsl:when>     
                    <xsl:when test="$nameUPPER='AGNT'">Agent</xsl:when> 
                    <xsl:when test="$nameUPPER='SUPVR'">Supervisor</xsl:when>   
                    <xsl:when test="$nameUPPER='SAIC'">SAIC</xsl:when>               
                    <xsl:when test="$nameUPPER='RAIC'">RAIC</xsl:when> 
                    <xsl:when test="$nameUPPER='JTTF'">JTTF</xsl:when>              
                    <xsl:when test="$nameUPPER='VIOLNC'">Violence</xsl:when>          
                    <xsl:when test="$nameUPPER='CLTRL'">Collateral</xsl:when>   
                    <xsl:when test="$nameUPPER='RQST'">Request</xsl:when>   
                    <xsl:when test="$nameUPPER='ROI'">ROI</xsl:when>  
                    <xsl:when test="$nameUPPER='RPT'">Report</xsl:when>   
                    <xsl:when test="$nameUPPER='RCRD'">Record</xsl:when>                       
                    <xsl:when test="$nameUPPER='RQST'">Request</xsl:when>                                   
                    <xsl:when test="$nameUPPER='AVIONC'">Avionic</xsl:when>
                    <xsl:when test="$nameUPPER='ARCRFT'">Aircraft</xsl:when>  
                    <xsl:when test="$nameUPPER='CMPLTN'">Completion</xsl:when> 
                    <xsl:when test="$nameUPPER='NCDNT'">Incident</xsl:when>    
                    <xsl:when test="$nameUPPER='PG'">Page</xsl:when>         
                    <xsl:when test="$nameUPPER='NRTV'">Narrative</xsl:when>  
                    <xsl:when test="$nameUPPER='RLTD'">Related</xsl:when>
                    <xsl:when test="$nameUPPER='CBP'">CBP</xsl:when>                       
                    <xsl:when test="$nameUPPER='SEVIS'">SEVIS</xsl:when>
                    <xsl:when test="$nameUPPER='CRMNL'">Criminal</xsl:when>  
                    <xsl:when test="$nameUPPER='AFLTN'">Affiliation</xsl:when>          
                    <xsl:when test="$nameUPPER='ETHNCTY'">Ethnicity</xsl:when>  
                    <xsl:when test="$nameUPPER='HAR'">Hair</xsl:when>          
                    <xsl:when test="$nameUPPER='TBL'">Table</xsl:when>    
                    <xsl:when test="$nameUPPER='FLD'">Field</xsl:when>  
                    <xsl:when test="$nameUPPER='SUBJ'">Subject</xsl:when> 
                    <xsl:when test="$nameUPPER='MISC'">Miscellaneous</xsl:when>  
                    <xsl:when test="$nameUPPER='ALT'">Alternate</xsl:when>       
                    <xsl:when test="$nameUPPER='OWNR'">Owner</xsl:when>      
                    <xsl:when test="$nameUPPER='ST'">State</xsl:when>                    
                    <xsl:when test="$nameUPPER='SUPP'">Support</xsl:when>
                    <xsl:when test="$nameUPPER='NCIC'">NCIC</xsl:when>
                    <xsl:when test="$nameUPPER='OCDETF'">OCDETF</xsl:when>
                    <xsl:when test="$nameUPPER='OBGYN'">OBGYN</xsl:when>
                    <xsl:when test="$nameUPPER='LAPR'">LAPR</xsl:when>
                    <xsl:when test="$nameUPPER='ISO'">ISO</xsl:when>
                    <xsl:when test="$nameUPPER='IIS'">IIS</xsl:when>
                    <xsl:when test="$nameUPPER='SSN'">SSN</xsl:when>
                    <xsl:when test="$nameUPPER='US'">US</xsl:when>
                    <xsl:when test="$nameUPPER='USA'">USA</xsl:when>
                    <xsl:when test="$nameUPPER='CATS'">CATS</xsl:when>
                    <xsl:when test="$nameUPPER='DUNS'">DUNS</xsl:when>
                    <xsl:when test="$nameUPPER='FEMA'">FEMA</xsl:when>
                    <xsl:when test="$nameUPPER='CNTRMSR'">Countermeasure</xsl:when>
                    <xsl:when test="$nameUPPER='PCKG'">Package</xsl:when>
                    <xsl:when test="$nameUPPER='CONSEQ'">Consequence</xsl:when>
                    <xsl:when test="$nameUPPER='RCNCLTN'">Reconciliation</xsl:when>
                    <xsl:when test="$nameUPPER='SBJT'">Subject</xsl:when>
                    <xsl:when test="$nameUPPER='PSPRT'">Passport</xsl:when>                    
                    <xsl:when test="$nameUPPER='PILT'">Pilot</xsl:when>
                    <xsl:when test="$nameUPPER='REGN'">Region</xsl:when>     
                    <xsl:when test="$nameUPPER='RSDNT'">Resident</xsl:when>                    
                    <xsl:when test="$nameUPPER='OCURNC'">Occurrence</xsl:when>   
                    <xsl:when test="$nameUPPER='POE'">PortOfEntry</xsl:when>  
                    <xsl:when test="$nameUPPER='PRIORT'">Priority</xsl:when> 
                    <xsl:when test="$nameUPPER='OTH'">Other</xsl:when>         
                    <xsl:when test="$nameUPPER='CRTY'">Country</xsl:when>
                    <xsl:when test="$nameUPPER='CTZNSHP'">Citizenship</xsl:when>    
                    <xsl:when test="$nameUPPER='HGT'">Height</xsl:when>   
                    <xsl:when test="$nameUPPER='WGT'">Weight</xsl:when> 
                    <xsl:when test="$nameUPPER='PRSN'">Person</xsl:when>   
                    <xsl:when test="$nameUPPER='CNTCT'">Contact</xsl:when>   
                    <xsl:when test="$nameUPPER='SPCL'">Special</xsl:when>
                    <xsl:when test="$nameUPPER='RMK'">Remark</xsl:when>   
                    <xsl:when test="$nameUPPER='PRMRY'">Primary</xsl:when>    
                    <xsl:when test="$nameUPPER='INSTRCTN'">Instruction</xsl:when>                       
                    <xsl:when test="$nameUPPER='INSP'">Inspect</xsl:when>
                    <xsl:when test="$nameUPPER='ASGN'">Assignment</xsl:when>
                    <xsl:when test="$nameUPPER='MSG'">Message</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='MSGID'">MessageId</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='MSGTYPE'">MessageType</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='FREETEXT'">FreeText</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='DATIME'">DateTime</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='UM'">UnitofMeasure</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='RGEO'">ResourceGeo</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='EGEO'">EventGeo</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='EVAC'">Evacuation</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='LON'">Longitude</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='LAT'">Latitude</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='ASSESSMNT'">Assessment</xsl:when><!-- CEN -->
                    <xsl:when test="$nameUPPER='SECLASS'">SecurityClass</xsl:when>
                    <xsl:when test="$nameUPPER='ESIGNATURE'">ESignature</xsl:when>
                    <xsl:when test="$nameUPPER='FINANCL'">Financial</xsl:when> 
                    <xsl:when test="$nameUPPER='NOTFD'">Notified</xsl:when> 
                    <xsl:when test="$nameUPPER='DTE' and($nextname='')">Date</xsl:when>
                    <xsl:when test="$nameUPPER='REC' and(upper-case($nextname)='KEY')">Record</xsl:when>
                    <xsl:when test="$nameUPPER='GRD' and(upper-case($nextname)='POST')">Guard</xsl:when>
                    <xsl:when test="$nameUPPER='VER' and(upper-case($nextname)='NUMBER')">Version</xsl:when>
                    <xsl:when test="starts-with($nameUPPER,'TBLDG')"><xsl:value-of select="$nameUPPER"/></xsl:when>
                    <xsl:when test="$nameUPPER='RO' and($nextname='BA54')">RO-</xsl:when>
                    <xsl:when test="$nameUPPER='RO' and($nextname='BA61')">RO-</xsl:when>
                    <xsl:when test="$nameUPPER='CO' and($nextname='BA54')">CO-</xsl:when>
                    <xsl:when test="$nameUPPER='CO' and($nextname='BA61')">CO-</xsl:when>  
                    <xsl:when test="$nameUPPER='ADA'">ADA</xsl:when>
                    <xsl:when test="$nameUPPER='ANSI'">ANSI</xsl:when>
                    <xsl:when test="$nameUPPER='BA54'">BA54</xsl:when>
                    <xsl:when test="$nameUPPER='BA61'">BA61</xsl:when>
                    <xsl:when test="$nameUPPER='BIA'">BIA</xsl:when>
                    <xsl:when test="$nameUPPER='BINAT'">BINAT</xsl:when>
                    <xsl:when test="$nameUPPER='BOC'">BOC</xsl:when>
                    <xsl:when test="$nameUPPER='BSC'">BSC</xsl:when>
                    <xsl:when test="$nameUPPER='COOP'">COOP</xsl:when>
                    <xsl:when test="$nameUPPER='CDC'">CDC</xsl:when>
                    <xsl:when test="$nameUPPER='CIP'">CIP</xsl:when>
                    <xsl:when test="$nameUPPER='CPT'">CPT</xsl:when>
                    <xsl:when test="$nameUPPER='DCO'">DCO</xsl:when>
                    <xsl:when test="$nameUPPER='DOJ'">DOJ</xsl:when>
                    <xsl:when test="$nameUPPER='DOT'">DOT</xsl:when>
                    <xsl:when test="$nameUPPER='DSO'">DSO</xsl:when>
                    <xsl:when test="$nameUPPER='EPA'">EPA</xsl:when>
                    <xsl:when test="$nameUPPER='EV'">EV</xsl:when>
                    <xsl:when test="$nameUPPER='EIS'">EIS</xsl:when>
                    <xsl:when test="$nameUPPER='FFMS'">FFMS</xsl:when>
                    <xsl:when test="$nameUPPER='HQ'">HQ</xsl:when>
                    <xsl:when test="$nameUPPER='ORI'">ORI</xsl:when>
                    <xsl:when test="$nameUPPER='SBA'">SBA</xsl:when>
                    <xsl:when test="$nameUPPER='UIC'">UIC</xsl:when>
                    <xsl:when test="$nameUPPER='FTTTF'">FTTTF</xsl:when>
                    <xsl:when test="$nameUPPER='LTC'">LTC</xsl:when>
                    <xsl:when test="$nameUPPER='HSPD12'">HSPD12</xsl:when>
                    <xsl:when test="$nameUPPER='DIBRS'">DIBRS</xsl:when> 
                    <xsl:when test="$nameUPPER='NAC'">NAC</xsl:when>
                    <xsl:when test="$nameUPPER='NIBRS'">NIBRS</xsl:when>
                    <xsl:when test="$nameUPPER='NNBIS'">NNBIS</xsl:when> 
                    <xsl:when test="$nameUPPER='NOLLE'">NOLLE</xsl:when>
                    <xsl:when test="$nameUPPER='NSEERS'">NSEERS</xsl:when>
                    <xsl:when test="$nameUPPER='OPM'">OPM</xsl:when>
                    <xsl:when test="$nameUPPER='INS'">INS</xsl:when>
                    <xsl:when test="$nameUPPER='IRP'">IRP</xsl:when>
                    <xsl:when test="$nameUPPER='POC'">POC</xsl:when>
                    <xsl:when test="$nameUPPER='FSRM'">FSRM</xsl:when>
                    <xsl:when test="$nameUPPER='RAMP'">RAMP</xsl:when>
                    <xsl:when test="$nameUPPER='FP'">FP</xsl:when>
                    <xsl:when test="$nameUPPER='FPS'">FPS</xsl:when>
                    <xsl:when test="$nameUPPER='FSA'">FSA</xsl:when>
                    <xsl:when test="$nameUPPER='SA'">SA</xsl:when>
                    <xsl:when test="$nameUPPER='MOE'">MOE</xsl:when>
                    <xsl:when test="$nameUPPER='STAR'">STAR</xsl:when>
                    <xsl:when test="$nameUPPER='STS'">STS</xsl:when>
                    <xsl:when test="$nameUPPER='UCR'">UCR</xsl:when>
                    <xsl:when test="$nameUPPER='GSA'">GSA</xsl:when>
                    <xsl:when test="$nameUPPER='PDF'">PDF</xsl:when>
                    <xsl:when test="$nameUPPER='PHN'">Phone</xsl:when>
                    <xsl:when test="$nameUPPER='PRRTY'">Priority</xsl:when>
                    <xsl:when test="$nameUPPER='VULN'">Vulnerability</xsl:when>
                    <xsl:when test="$nameUPPER='VWPP'">VWPP</xsl:when>
                    <xsl:when test="$nameUPPER='ZIP'">ZIP</xsl:when>
                    <xsl:when test="$nameUPPER='ZADVYDAS'">ZADVYDAS</xsl:when>
                    <xsl:when test="$nameUPPER='COC'">CountryOfCitizenship</xsl:when>
                    <xsl:when test="$nameUPPER='COB'">CountryOfBirth</xsl:when>
                    <xsl:when test="$nameUPPER='DOB'">DateOfBirth</xsl:when>
                    
                    <xsl:otherwise><xsl:value-of select="camed:correctSpelling($name)"/></xsl:otherwise>
                </xsl:choose>
   
            </xsl:when>
            <xsl:otherwise><xsl:value-of select="camed:correctSpelling($name)"/></xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <xsl:function name="camed:WordMatch">
        <xsl:param name="name"/>
        <xsl:param name="mode"/>
        
        <xsl:choose>
            <xsl:when test="$mode='NIEM'">
                <xsl:choose>
                    <xsl:when test="$name='URL'">true</xsl:when>
                    <xsl:when test="$name='URI'">true</xsl:when>
                    <xsl:when test="$name='URN'">true</xsl:when>
                    <xsl:when test="$name='UIC'">true</xsl:when>
                    <xsl:when test="$name='CRC'">true</xsl:when>
                    <xsl:when test="$name='X509SKI'">true</xsl:when>
                    <xsl:when test="$name='X509CRL'">true</xsl:when>
                    <xsl:when test="$name='DCO'">true</xsl:when>
                    <xsl:when test="$name='ICE'">true</xsl:when>
                    <xsl:when test="$name='FBI'">true</xsl:when>
                    <xsl:when test="$name='STS'">true</xsl:when>
                    <xsl:when test="$name='RAMP'">true</xsl:when>
                    <xsl:when test="$name='SEVIS'">true</xsl:when>
                    <xsl:otherwise>false</xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <xsl:function name="camed:correctSpelling">
        <xsl:param name="word"/>
        <xsl:variable name="wordLOWER" select="lower-case($word)"/>
        <xsl:choose>
            <xsl:when test="$wordsList//word[@wrong=$wordLOWER]"><xsl:value-of select="$wordsList//word[@wrong=$wordLOWER]/@correct[1]"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$word"/></xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <xsl:function name="camed:setSpellings">
        <words>
            <word correct="Acceptable" wrong="acceptible"/>
            <word correct="Accidently" wrong="accidentaly"/>
            <word correct="Accidently" wrong="accidentally"/>
            <word correct="Accommodate" wrong="accomodate"/>
            <word correct="Accommodate" wrong="acommodate"/>
            <word correct="Acknowledge" wrong="acknowlege"/>
            <word correct="Acknowledge" wrong="aknowledge"/>
            <word correct="Acquaintance" wrong="acquaintence"/>
            <word correct="Acquaintance" wrong="aquaintance"/>
            <word correct="Acquire" wrong="aquire"/>
            <word correct="Acquire" wrong="adquire"/>
            <word correct="Acquit" wrong="aquit"/>
            <word correct="Acreage" wrong="acrage"/>
            <word correct="Acreage" wrong="acerage"/>
            <word correct="Address" wrong="adress"/>
            <word correct="Advertise" wrong="advertize"/>
            <word correct="Advice" wrong="advise"/>
            <word correct="Advisable" wrong="adviseable"/>
            <word correct="Advisable" wrong="advizable"/>
            <word correct="Alcohol" wrong="alchohol"/>
            <word correct="Allege" wrong="alege"/>
            <word correct="Alleged" wrong="aleged"/>
            <word correct="Allege" wrong="allage"/>
            <word correct="Alleged" wrong="allaged"/>
            <word correct="Allegiance" wrong="allegaince"/>
            <word correct="Allegiance" wrong="allegience"/>
            <word correct="Allegiance" wrong="alegiance"/>
            <word correct="Amateur" wrong="amatuer"/>
            <word correct="Amateur" wrong="amature"/>
            <word correct="Amend" wrong="ammend"/>
            <word correct="Among" wrong="amung"/>
            <word correct="Annually" wrong="anually"/>
            <word correct="Annually" wrong="annualy"/>
            <word correct="Apparent" wrong="apparant"/>
            <word correct="Apparent" wrong="aparent"/>
            <word correct="Arctic" wrong="artic"/>
            <word correct="Argument" wrong="arguement"/>
            <word correct="Athlete" wrong="athelete"/>
            <word correct="Average" wrong="avrage"/>
            <word correct="Average" wrong="averege"/>
            <word correct="Balance" wrong="ballance"/>
            <word correct="Balance" wrong="balence"/>
            <word correct="Basically" wrong="basicly"/>
            <word correct="Before" wrong="befor"/>
            <word correct="Beginning" wrong="begining"/>
            <word correct="Believe" wrong="beleive"/>
            <word correct="Benefit" wrong="benifit"/>
            <word correct="Business" wrong="bisness"/>
            <word correct="Business" wrong="bussiness"/>
            <word correct="Business" wrong="bizness"/>
            <word correct="Business" wrong="buisness"/>
            <word correct="Calendar" wrong="calender"/>
            <word correct="Camouflage" wrong="camoflage"/>
            <word correct="Camouflage" wrong="camoflague"/>
            <word correct="Career" wrong="carrer"/>
            <word correct="Category" wrong="catagory"/>
            <word correct="Caught" wrong="cauhgt"/>
            <word correct="Caught" wrong="caugt"/>
            <word correct="Ceiling" wrong="cieling"/>
            <word correct="Cemetery" wrong="cemetary"/>
            <word correct="Cemetery" wrong="cematery"/>
            <word correct="Certain" wrong="certin"/>
            <word correct="Cesium" wrong="caesium"/>
            <word correct="Changeable" wrong="changable"/>
            <word correct="Chief" wrong="cheif"/>
            <word correct="Citizen" wrong="citezen"/>
            <word correct="Colleague" wrong="collaegue"/>
            <word correct="Colleague" wrong="collegue"/>
            <word correct="Column" wrong="colum"/>
            <word correct="Coming" wrong="comming"/>
            <word correct="Committed" wrong="commited"/>
            <word correct="Committed" wrong="comitted"/>
            <word correct="Commit" wrong="comit"/>
            <word correct="Commit" wrong="comitt"/>
            <word correct="Competition" wrong="compitition"/>
            <word correct="Concede" wrong="conceed"/>
            <word correct="Conscientious" wrong="consciencious"/>
            <word correct="Conscious" wrong="concious"/>
            <word correct="Conscious" wrong="consious"/>
            <word correct="Census" wrong="sensus"/>
            <word correct="Convenience" wrong="conveniance"/>
            <word correct="Decide" wrong="dicide"/>
            <word correct="Definite" wrong="definate"/>
            <word correct="Definite" wrong="definit"/>
            <word correct="Definitely" wrong="definitly"/>
            <word correct="Definitely" wrong="definately"/>
            <word correct="Definitely" wrong="defiantly"/>
            <word correct="Deposit" wrong="deposite"/>
            <word correct="Discretionary" wrong="descretionary"/>
            <word correct="Develop" wrong="develope"/>
            <word correct="Difference" wrong="diffrence"/>
            <word correct="During" wrong="durring"/>
            <word correct="Environment" wrong="enviroment"/>
            <word correct="Equipped" wrong="equiped"/>
            <word correct="Equipment" wrong="equiptment"/>
            <word correct="Exceed" wrong="excede"/>
            <word correct="Except" wrong="exsept"/>
            <word correct="Exercise" wrong="exercize"/>
            <word correct="Exercise" wrong="exersize"/>
            <word correct="Existence" wrong="existance"/>
            <word correct="Experience" wrong="experiance"/>
            <word correct="Experiment" wrong="experament"/>
            <word correct="Explanation" wrong="explaination"/>
            <word correct="Finally" wrong="finaly"/>
            <word correct="Foreign" wrong="foriegn"/>
            <word correct="Forward" wrong="foreward"/>
            <word correct="Friend" wrong="freind"/>
            <word correct="Fulfill" wrong="fullfil"/>
            <word correct="Fundamental" wrong="fundemental"/>
            <word correct="Gauge" wrong="guage"/>
            <word correct="Generally" wrong="generaly"/>
            <word correct="Generally" wrong="genrally"/>
            <word correct="General" wrong="genrall"/>
            <word correct="General" wrong="genral"/>
            <word correct="Government" wrong="goverment"/>
            <word correct="Grammar" wrong="grammer"/>
            <word correct="Guarantee" wrong="garantee"/>
            <word correct="Guarantee" wrong="garentee"/>
            <word correct="Guarantee" wrong="garanty"/>
            <word correct="Guidance" wrong="guidence"/>
            <word correct="Height" wrong="heighth"/>
            <word correct="Height" wrong="heigth"/>
            <word correct="Hierarchy" wrong="heirarchy"/>
            <word correct="Hygiene" wrong="hygene"/>
            <word correct="Hygiene" wrong="hygine"/>
            <word correct="Hygiene" wrong="higeine"/>
            <word correct="Hygiene" wrong="hygeine"/>
            <word correct="Identity" wrong="idenity"/>
            <word correct="Identity" wrong="identidy"/>
            <word correct="Imaginary" wrong="imaginery"/>
            <word correct="Imitate" wrong="immitat"/>
            <word correct="Imitate" wrong="immitate"/>
            <word correct="Imitation" wrong="immitation"/>
            <word correct="Imitation" wrong="imitashun"/>
            <word correct="Immediate" wrong="imediate"/>
            <word correct="Immediately" wrong="imediately"/>
            <word correct="Independent" wrong="independant"/>
            <word correct="Inoculate" wrong="innoculate"/>
            <word correct="Intelligence" wrong="inteligence"/>
            <word correct="Intelligence" wrong="intelligance"/>
            <word correct="Interpreter" wrong="interpretor"/>
            <word correct="Interrupt" wrong="interupt"/>
            <word correct="Irrelevant" wrong="irrelevent"/>
            <word correct="Island" wrong="iland"/>
            <word correct="Jewelry" wrong="jewelery"/>
            <word correct="Knowledge" wrong="knowlege"/>
            <word correct="Laboratory" wrong="labratory"/>
            <word correct="Length" wrong="lenght"/>
            <word correct="Liaison" wrong="liason"/>
            <word correct="Library" wrong="libary"/>
            <word correct="Library" wrong="liberry"/>
            <word correct="License" wrong="lisence"/>
            <word correct="License" wrong="licence"/>
            <word correct="Lightning" wrong="lightening"/>
            <word correct="Maintenance" wrong="maintenence"/>
            <word correct="Maintenance" wrong="maintnance"/>
            <word correct="Maneuver" wrong="manuever"/>
            <word correct="Maneuver" wrong="manuver"/>
            <word correct="Marriage" wrong="marrige"/>
            <word correct="Mathematics" wrong="mathmatics"/>
            <word correct="Medicine" wrong="medcine"/>
            <word correct="Miniature" wrong="miniture"/>
            <word correct="Minute" wrong="minite"/>
            <word correct="Mission" wrong="mmision"/>
            <word correct="Mission" wrong="mision"/>
            <word correct="Necessary" wrong="necessery"/>
            <word correct="Necessary" wrong="neccessary"/>
            <word correct="Neighbor" wrong="nieghbor"/>
            <word correct="Neither" wrong="niether"/>
            <word correct="Noticeable" wrong="noticable"/>
            <word correct="Occasion" wrong="occassion"/>
            <word correct="Occasionally" wrong="occassionally"/>
            <word correct="Occasionally" wrong="occasionaly"/>
            <word correct="Occurrence" wrong="occurrance"/>
            <word correct="Occurrence" wrong="occurence"/>
            <word correct="Occur" wrong="ocure"/>
            <word correct="Occur" wrong="ocur"/>
            <word correct="Occur" wrong="ocurr"/>
            <word correct="Occurred" wrong="occured"/>
            <word correct="Official" wrong="oficial"/>
            <word correct="Often" wrong="offen"/>
            <word correct="Omission" wrong="ommision"/>
            <word correct="Omission" wrong="omision"/>
            <word correct="Operate" wrong="oprate"/>
            <word correct="Opportunity" wrong="oppurtunity"/>
            <word correct="Original" wrong="orignal"/>
            <word correct="Paid" wrong="payed"/>
            <word correct="Parallel" wrong="parrallel"/>
            <word correct="Parallel" wrong="parellel"/>
            <word correct="Parliament" wrong="parliment"/>
            <word correct="Particular" wrong="particur"/>
            <word correct="Particularly" wrong="particurly"/>
            <word correct="Pastime" wrong="passtime"/>
            <word correct="Pastime" wrong="pasttime"/>
            <word correct="Peculiar" wrong="peculier"/>
            <word correct="Perceive" wrong="percieve"/>
            <word correct="Permanent" wrong="pernament"/>
            <word correct="Personally" wrong="personaly"/>
            <word correct="Personnel" wrong="personell"/>
            <word correct="Personnel" wrong="personel"/>
            <word correct="Picture" wrong="pichure"/>
            <word correct="Picture" wrong="pitcher"/>
            <word correct="Piece" wrong="peice"/>
            <word correct="Planning" wrong="planing"/>
            <word correct="Political" wrong="pollitical"/>
            <word correct="Possession" wrong="posession"/>
            <word correct="Possession" wrong="possesion"/>
            <word correct="Practical" wrong="practicle"/>
            <word correct=" precede" wrong="Preceed"/>
            <word correct="Prejudice" wrong="predjudice"/>
            <word correct="Presence" wrong="presance"/>
            <word correct="Principle" wrong="principal"/>
            <word correct="Privilege" wrong="privelege"/>
            <word correct="Privilege" wrong="priviledge"/>
            <word correct="Probably" wrong="probly"/>
            <word correct="Professional" wrong="proffesional"/>
            <word correct="Professor" wrong="professer"/>
            <word correct="Promise" wrong="promiss"/>
            <word correct="Pronunciation" wrong="pronounciation"/>
            <word correct="Proof" wrong="prufe"/>
            <word correct="Psychology" wrong="psycology"/>
            <word correct="Publicly" wrong="publically"/>
            <word correct="Quantity" wrong="quanity"/>
            <word correct="Quarantine" wrong="quarentine"/>
            <word correct="Quarter" wrong="Korter"/>
            <word correct="Queue" wrong="que"/>
            <word correct="Questionnaire" wrong="questionaire"/>
            <word correct="Questionnaire" wrong="questionnair"/>
            <word correct="Readable" wrong="readible"/>
            <word correct="Realize" wrong="relize"/>
            <word correct="Receive" wrong="recieve"/>
            <word correct="Receipt" wrong="reciept"/>
            <word correct="Recognize" wrong="reconize"/>
            <word correct="Recognize" wrong="reckonize"/>
            <word correct="Recommend" wrong="recomend"/>
            <word correct="Recommend" wrong="reccommend"/>
            <word correct="Recommended" wrong="recomended"/>
            <word correct="Recommended" wrong="reccommended"/>
            <word correct="Referred" wrong="refered"/>
            <word correct="Reference" wrong="referance"/>
            <word correct="Reference" wrong="refrence"/>
            <word correct="Relevant" wrong="relevent"/>
            <word correct="Relevant" wrong="revelant"/>
            <word correct="Religious" wrong="religous"/>
            <word correct="Religious" wrong="religius"/>
            <word correct="Religion" wrong="religon"/>
            <word correct="Religion" wrong="religin"/>
            <word correct="Repetition" wrong="repitition"/>
            <word correct="Restaurant" wrong="restarant"/>
            <word correct="Safety" wrong="saftey"/>
            <word correct="Safety" wrong="safty"/>
            <word correct="Secretary" wrong="secratary"/>
            <word correct="Secretary" wrong="secretery"/>
            <word correct="Seize" wrong="sieze"/>
            <word correct="Separate" wrong="seperate"/>
            <word correct="Sergeant" wrong="sargent"/>
            <word correct="Similar" wrong="similer"/>
            <word correct="Similar" wrong="simmilar"/>
            <word correct="Similar" wrong="simular"/>
            <word correct="Sincerely" wrong="sinceerly"/>
            <word correct="Soldier" wrong="solider"/>
            <word correct="Speech" wrong="speach"/>
            <word correct="Speech" wrong="speeche"/>
            <word correct="Stopping" wrong="stoping"/>
            <word correct="Strength" wrong="strenght"/>
            <word correct="Succeed" wrong="succede"/>
            <word correct="Successful" wrong="succesful"/>
            <word correct="Successful" wrong="successfull"/>
            <word correct="Successful" wrong="sucessful"/>
            <word correct="Supersede" wrong="supercede"/>
            <word correct="Temperature" wrong="temperture"/>
            <word correct="Temporary" wrong="temprary"/>
            <word correct="Tomorrow" wrong="tommorrow"/>
            <word correct="Until" wrong="untill"/>
            <word correct="Unusual" wrong="unuseual"/>
            <word correct="Using" wrong="useing"/>
            <word correct="Usually" wrong="usualy"/>
            <word correct="Vacuum" wrong="vaccuum"/>
            <word correct="Vacuum" wrong="vacume"/>
            <word correct="Vegetarian" wrong="vegatarian"/>
            <word correct="Vehicle" wrong="vehical"/>
            <word correct="Village" wrong="villege"/>
            <word correct="Weather" wrong="wether"/>
            <word correct="Welcome" wrong="wellcome"/>
            <word correct="Welfare" wrong="wellfare"/>
            <word correct="Welfare" wrong="welfair"/>
            <word correct="Withhold" wrong="withold"/>
            <word correct="Writing" wrong="writting"/>
            <word correct="Writing" wrong="writeing"/>
            <!-- Application words -->
            <word wrong="certifcation" correct="Certification"/>
            <word wrong="financl" correct="Financial"/>
            <word wrong="setence" correct="Sentence"/>
            <word wrong="notifd" correct="Notified"/>
            <word wrong="asignd" correct="Assigned"/>
            <word wrong="compltd" correct="Completed"/>
            <word wrong="finl" correct="Final"/>
            <word wrong="coordinatr" correct="Coordinator"/>
            <word wrong="rsult" correct="Result"/>
            <word wrong="setenced" correct="Sentenced"/>
            <word wrong="quatity" correct="Quantity"/>
            <word wrong="enforcment" correct="Enforcement"/>
            <word wrong="siezed" correct="Seized"/>
            <word wrong="assesstype" correct="Assessment"/>
            <word wrong="tagencybu" correct="AgencyBureau"/>
            <word wrong="catrgory" correct="Category"/>
            <word wrong="countermeasurefunding" correct="CountermeasureFunding"/>
            <word wrong="emrgency" correct="Emergency"/>
            <word wrong="orginal" correct="Original"/>
            <word wrong="descrition" correct="Description"/>
            <word wrong="indentifier" correct="Identifier"/>
            <word wrong="strcture" correct="Structure"/>
            <word wrong="decisn" correct="Decision"/>
            <word wrong="reas" correct="Reason"/>
            <word wrong="notif" correct="Notify"/>
            <word wrong="adjstd" correct="Adjusted"/>
            <word wrong="arrngmnt" correct="Arrangement"/>
            <word wrong="Prrty" correct="Priority"/>
            <xsl:if test="$rules//camed:annotation/camed:documentation[@type='Spelling']">
                <xsl:message> Spelling from, to:</xsl:message>
            </xsl:if>
            <xsl:variable name="wordsList">
            <xsl:for-each select="$rules//camed:annotation/camed:documentation[@type='Spelling']">
                <xsl:if test="contains(.,'=')">
                    <xsl:variable name="wrong" select="substring-before(.,'=')"/>
                    <xsl:variable name="correct" select="substring-after(.,'=')"/>
                    <!-- add to spellings list -->
                    <word wrong="{$wrong}" correct="{$correct}"/>
                </xsl:if>
            </xsl:for-each>
            </xsl:variable>
            <xsl:copy-of select="$wordsList"/>
            <!-- record details in console log -->
            <xsl:message>
                <xsl:for-each select="$wordsList/word"> <xsl:value-of select="@wrong"/>, <xsl:value-of select="@correct"/> </xsl:for-each>
            </xsl:message>
        </words>
    </xsl:function>

</xsl:stylesheet>
