<?xml version="1.0" encoding="UTF-8"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:lexs="http://usdoj.gov/leisp/lexs/3.1" 
    xmlns:w="http://niem.gov/niem/wantlist/1" 
    exclude-result-prefixes="xs xsd" version="2.0">

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: David RR Webber, April, 2008 -->

    <!-- 

Purpose : Take any XML instance and create CAM wantlist
Original : 04/17/2008 
Initials : drrw 
CAM ver : 1.1 
Description : Part of a set of tools to ease alignment of XSD based validations and documentation 
and CAM templates. 
How to use :  The runMode and depth parameters are provided to control the level of detail output.  For really large schemas you may prefer to selectively increase the level of detail - initial settings should withNoAttr and depth 4 or 5.
                      Subsequent runs can then increase detail - after re-generating examples using xsd2example filtered to exclude unwanted parts.
Updated - Inits - Change 
04/17/08 drrw Initial release 
-->
    <xsl:param name="runMode" select="'withNoAttr'"/>
    <xsl:param name="depth" select="3"/>
    
    <xsl:output indent="yes"/>

    <xsl:template match="/">

        <xsl:variable name="ns-prefix">
            <xsl:value-of select="./@targetNamespace"/>
        </xsl:variable>        

        <w:WantList xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xmlns:w="http://niem.gov/niem/wantlist/1"
            xmlns:as="http://www.oasis-open.org/committees/cam"
            w:release="1.0"
            w:product="NIEM/CAM" w:runMode="{$runMode}" w:depth="{$depth}"> 
            
            <xsl:for-each select="*">
                <!-- parses the entire XML input -->
                <xsl:element name="Group">
                    <xsl:attribute name="w:n"><xsl:value-of select="concat(name(../.),'\',name(.))"/></xsl:attribute>
                    <xsl:attribute name="w:isXclude"><xsl:value-of select="'0'"/></xsl:attribute>
                    <xsl:apply-templates select="./@*" mode="map"/>
                    <xsl:apply-templates mode="map" select="*"/>                  
                </xsl:element>
            </xsl:for-each>
           
        </w:WantList> 
    </xsl:template>
    
    <xsl:template match="node()/child::*" mode="map">

        <xsl:element name="Group">
            <xsl:attribute name="w:n"><xsl:value-of select="concat(name(../.),'\',name(.))"/></xsl:attribute>
            <xsl:attribute name="w:isXclude"><xsl:value-of select="'0'"/></xsl:attribute>
            <xsl:apply-templates select="./@*" mode="map"/>
            <xsl:for-each select="*">
                <xsl:choose>
                    <xsl:when test="count(./descendant::node()) > 1">
                        <xsl:variable name="chkdepth">
                            <xsl:choose>
                                <xsl:when test="count(../../../../../../../parent::*) &gt; 0">7</xsl:when>
                                <xsl:when test="count(../../../../../../parent::*) &gt; 0">6</xsl:when>
                                <xsl:when test="count(../../../../../parent::*) &gt; 0">5</xsl:when>
                                <xsl:when test="count(../../../../parent::*) &gt; 0">4</xsl:when>
                                <xsl:when test="count(../../../parent::*) &gt; 0">3</xsl:when>
                                <xsl:when test="count(../../parent::*) &gt; 0">2</xsl:when>
                                <xsl:when test="count(../parent::*) &gt; 0">1</xsl:when>
                                <xsl:otherwise>8</xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:if test="$depth &gt; $chkdepth">
                            <xsl:apply-templates mode="map" select="."/>
                        </xsl:if>
                        <xsl:apply-templates mode="map" select="./@*"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:element name="Element">
                            <xsl:attribute name="w:n"><xsl:value-of select="concat(name(../../.),'\',name(../.),'\',name(.))"/></xsl:attribute>
                            <xsl:attribute name="w:isXclude"><xsl:value-of select="'0'"/></xsl:attribute>
                            <xsl:apply-templates  select="./@*" mode="map"/>
                        </xsl:element>
                    </xsl:otherwise>
                </xsl:choose>   
            </xsl:for-each>
         </xsl:element>
    </xsl:template>   
    
    <xsl:template match="attribute::node()" mode="map">
        <xsl:if test="contains($runMode,'withAttr')">
        <xsl:element name="Attribute">
            <xsl:attribute name="w:n"><xsl:value-of select="concat(name(../../.),'\',name(../.),'\@',name(.))"/></xsl:attribute>
            <!-- xsl:attribute name="w:v"><xsl:value-of select="."/></xsl:attribute -->
            <xsl:attribute name="w:isXclude"><xsl:value-of select="'0'"/></xsl:attribute>
        </xsl:element>
       </xsl:if>       
    </xsl:template>

</xsl:stylesheet>
